/** 
 * @Version $Rev: 168 $
 * @Author M.Sekido
 * @Date  $Date: 2015-09-13 21:41:11 +0900 (日, 13  9月 2015) $
 * $Id: k5vsi-vssp-dim.cpp 168 2015-09-13 12:41:11Z vlbi $ 
 */


#ifndef LINUX
#define LINUX
#endif
#ifndef _LARGEFILE_SOURCE 
#define _LARGEFILE_SOURCE
#endif 
#define _LARGEFILE64_SOURCE

#include <iostream>
#include <string>
using namespace std;

extern "C"{
  //#include <getopt.h>
}

#include "InVSI_mdl.hpp"
// #include "VDIF_Proto.hpp"
#include "VLBI_Frame.hpp"
#include "FileDumper.hpp" // modify 2015/3/9 miya Dumper->FileDumper
#include "Trans_info.hpp"
#include "Raw_Proto.hpp"
#include "conf_utl.hpp"

#define DEBUG(x) 

int VirDumper::dth_number=0; // modify 2015/3/9 miya Dumper->VirDumper
int Rate_Meas::thread_number=0;
/** General Configulation data */
Config_set vdif_conf;

char *me;  // My name

void usage(){
  printf("Usage: %s [options]\n",me);
  printf("  -s <Byte>        :  UDP block size\n");
  //  printf("  -f <vdif/m5b/k5> :  Data Frame type. Default(vdif)\n");
  printf("  -t <xs|xm|xh>    :  time length to sent\n");
  printf("  -p <PORT>        :  Port to be used\n");
  //  printf("  -q <SUDP/VDIF>:  Appl-Trans Protocol. (default SUDP/udp)\n");
  printf("  -Q <udp/tcp/file> :  Protocol for VDIF transfer. default:file output\n");
  printf("  -a <HOSTNAME>    :  Remote host name\n");
  printf("  -m <'xxMsps/xbit/xxch'>: Data mode\n");
  printf("        Msps -> [04Msps|08Msps|16Msps|32Msps|64Msps]\n");
  printf("        bit  -> 2bit only                       \n");
  printf("        ch   -> [16ch|08ch|04ch]                \n");
  printf("  -o <out file name>: Writing data to a file. \n");
  printf("      example: -o file1:file2. If  #filenames < #group, \n");
  printf("         then -a,b,c,d will be used.ex.) file1-a.k5 file1-b.k5 file1-c.k5 \n");
  printf("  -g <igrp> : Group number 0-32/nch, default(0).\n");
  printf("      example: -g 0:1:2:3 , -g 0:1:2.\n");
  printf("  -d debug output to /tmp \n");
  printf("  -h display this list.\n");
  exit(1);
}

int main (int argc, char * const argv[]) {
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  
  /// Device Info Setup 
  string mode("08Msps/2bit/4ch");

  vector <string> hostname;
  //  string proto="udp"; 
  //  VLBI_Proto::TransProtocol_t  proto =VLBI_Proto::UDP_TX;
  int port=DEFAULT_PORT;
  int mtu  = DEFAULT_MTU;
  if(argc < 2) usage();
  vector <string> outfname;
  vector <int> igrp;
  int tspan = 11;
  char buf[80];
  TransInfo::TransProtocol_t tranp = TransInfo::FILE_IO;
  TransInfo::ApplProtocol_t appp = TransInfo::SUDP_Proto;
  bool debug = false;
  
  int k=0;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hs:t:p:a:m:o:g:q:Q:d")) { 
    case 'h':   usage();     break;
    case 'd':   debug = true;     break;
    case 'p':   port = atoi(optarg); break;
    case 'Q': 
      if(strncmp(optarg,"tcp",3)==0) tranp = TransInfo::TCP_IO;
      if(strncmp(optarg,"udp",3)==0) tranp = TransInfo::UDP_IO;
      break;
    case 'a':   {string work  = optarg; hostname.push_back(work);}  break;
    case 't':   tspan = Str2Tint(optarg); break;
    case 'g': { 
      if(strlen(optarg)>80){ cerr << "-g option too long\n"; exit(1);}
      strcpy(buf,optarg);
      char *ptr(0), *ptr0=buf;;
      int ig;
      while((ptr=strchr(ptr0,':')) !=NULL){
	*ptr='\0'; ig=atoi(ptr0); igrp.push_back(ig);
	ptr0 = ptr+1;
      }
      ig=atoi(ptr0); igrp.push_back(ig);
    }break;
    case 'm':    mode = optarg;   break;
    case 'o':{ 
      tranp = TransInfo::FILE_IO;
      string  work =optarg;
      size_t pos=0;
      while( (pos=work.find(":")) != work.npos){
	outfname.push_back(work.substr(0,pos));
	work = work.substr(pos+1);
      };
      outfname.push_back(work);  
    }break;
    case 's':    mtu = atoi(optarg); break;
    case '?':    usage(); break;
    }
  }
  if(igrp.size() ==0) igrp.push_back(0);
  
  if(outfname.size() != 0){
    appp = TransInfo::RAW_Proto;
  }else if( hostname.size() !=0){
    appp = TransInfo::SUDP_Proto;
  }else{
    cerr << "No Output (file/net) Specified\n";
    usage();
  }

  if(debug){
    string logdir = "/tmp/";
    string info_file = logdir + me + "_info.txt";
    string debug_file = logdir + me + "_dbg.txt";
    string error_file = logdir + me +"_error.txt";
    Logsystem::info_logger()->set_stream(info_file);
    Logsystem::debug_logger()->set_stream(debug_file);
    Logsystem::error_logger()->set_stream(error_file);
    
    Logsystem::debug_logger()->enable_logging();
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
  }

  DAS_Info dasinf;
  dasinf.SetMode(mode);
  
  K5_VSI k5vsi0;
  k5vsi0.Set_Mode(mode.c_str());
  //  k5vsi0.Set_Mode("32Msps/2bit/16ch");
  
  int ngrp = 32/dasinf.Nch()/dasinf.Qbit();
  if( (int)igrp.size() > ngrp){
    fprintf(stderr, "Error: available ngrp=%u for %s, but specified ngrp=%u\n",
	    ngrp, mode.c_str(), (unsigned int)igrp.size());
    return -1;
  }

  ngrp = (int)igrp.size();
  VLBI_Proto *vlbi_pro[ngrp];
  
  if(tranp == TransInfo::FILE_IO && ngrp != (int)outfname.size()){
    fprintf(stderr,
	    "Error Number of group(%u) must be the same with #output files(%d)\n",
	    ngrp, (unsigned int)outfname.size());
    usage();
  }
  if( tranp != TransInfo::FILE_IO && ngrp != (int)hostname.size()){
    fprintf(stderr,
	    "Error Number of group(%u) must be the same with #hostnames(%d)\n",
	    ngrp, (unsigned int)hostname.size());
    usage();
  }

  if(appp == TransInfo::SUDP_Proto)
    for(int i=0; i<ngrp; i++) vlbi_pro[i] = new SUDPProto(mtu);
  if(appp == TransInfo::RAW_Proto)
    for(int i=0; i<ngrp; i++) vlbi_pro[i] = new RAW_Proto();


  /// Setup protocol Output interface   
  //  VDIF_Proto vdif_pro[ngrp]; // Using VDIF Header and protocol 
  //   if(proto[0] == VLBI_Proto::VDIF_Prot)
  //     for(int i=0; i<ngrp; i++) vlbi_pro[i] = new VDIF_Proto(mtu);
  
  K5_VSI k5vsi[ngrp];  // K5VSI device module 
  //  for(int i=0; i<ngrp; i++) k5vsi[i].Set_Mode(mode.c_str());
  for(int i=0; i<ngrp; i++) k5vsi[i].Set_Mode("32Msps/2bit/16ch");
  
  
  switch(tranp){
  case VLBI_Proto::UDP_IO:
    //    if(proto.find("udp",0,3) != proto.npos){
    for(int j=0; j<ngrp; j++){
      vlbi_pro[j]->Set_mtu(mtu);
      vlbi_pro[j]->Set_Transprotocol(VLBI_Proto::UDP_IO);
      vlbi_pro[j]->Set_Applprotocol(VLBI_Proto::SUDP_Proto);
      vlbi_pro[j]->Create_sock(); 
      printf(" Data (%s) Output to %s by Net(UDP) \n", 
	     k5vsi[j].Mode().c_str(),hostname[j].c_str());
    }
    break;
  case VLBI_Proto::TCP_IO:
    //if(proto.find("tcp",0,3) != proto.npos){
    for(int j=0; j<ngrp; j++){
      vlbi_pro[j]->Set_mtu(mtu);
      vlbi_pro[j]->Set_Transprotocol(VLBI_Proto::TCP_IO);
      vlbi_pro[j]->Set_Applprotocol(VLBI_Proto::RAW_Proto);
      vlbi_pro[j]->Create_sock(); 
      printf(" Data (%s) Output to %s by Net(TCP) \n", 
	     k5vsi[j].Mode().c_str(),hostname[j].c_str());
    } break;
  case  VLBI_Proto::FILE_IO:
    //if(proto.find("file",0,4) != proto.npos){
    for(int j=0; j<ngrp; j++){
      vlbi_pro[j]->Set_Transprotocol(VLBI_Proto::FILE_IO);
      vlbi_pro[j]->Set_Applprotocol(VLBI_Proto::RAW_Proto);
      vlbi_pro[j]->Create_sock(); 
      printf(" Data (%s) Output to FILE %s \n", 
	     k5vsi[j].Mode().c_str(),outfname[j].c_str());
    }
    break;
  }
    //  }
  if(tranp == TransInfo::FILE_IO)  for(int j=0; j<ngrp; j++){
    vlbi_pro[j]->Set_Host(outfname[j].c_str());
    //    vlbi_pro[j]->Set_DataPort(port);
  }
  else  for(int j=0; j<ngrp; j++){
    vlbi_pro[j]->Set_Host(hostname[j].c_str());
    vlbi_pro[j]->Set_DataPort(port);
  }

  /// Setup data format/frame 
  //  VDIF_Frame vdif;
  K5VSSP32_Frame k5f[ngrp];
  FileDumper dumper[ngrp]; // modify 2015/3/9 miya Dumper->FileDumper

  for(int j=0; j<ngrp; j++){
    dasinf.SetFirstCh(igrp[j]*4);
    if(  k5f[j].Set_DasInfo(dasinf) <0){
      cerr << "Error in Setting Sampling parameters. This mode("
	   << k5vsi[j].Mode().c_str() << ") is not supported by K5VSSP32.\n";
      return -1;
    }
    //    k5vsi[j].Set_iGrp(igrp[j]);
    k5f[j].Set_StnameID("Kashim34","kb");
    //  k5f.Set_Nframe_sec(1);
    vlbi_pro[j]->Set_VlbiFrame(&k5f[j]);
    if( ! vlbi_pro[j]->Tx_Ready() ){
      if(appp == VLBI_Proto::SUDP_Proto) 
	cerr << "Error: SUDP-Protocol is not ready\n"; 
      else
	cerr << "Error: VDIF-Protocol is not ready\n"; 
      exit(1);
    };
    
    dumper[j].Set_Span(tspan);
    dumper[j].SetDasMode(VirDumper::USR_OUT_SET);// modify 2015/3/9 miya Dumper->VirDumper
    if(dumper[j].Set_IO(vlbi_pro[j], &k5vsi[j],j) <0){
      cerr << "Error Dumper::Set_IO()\n";
      return -1;
    }
    
    if(dumper[j].Ready()){
      fprintf(stderr, " Dump[%d] Starting.\n",j);
      dumper[j].start();
    }else{
      fprintf(stderr, " Dump[%d] Did not Started due to Ready() error.\n",j);
    }

  }

  for(int j=0; j<ngrp; j++){
    //  while(1){     ; }
    if(dumper[j].join() !=0){
      cerr << "dumper returned with error.\n";
    }else
      cerr << "dumper returned with success.\n";
  }

  
  return(0);
}

