
#ifndef __EXTSUBCH_HPP__
#define __EXTSUBCH_HPP__
/** Extracting Channel data class library
 * @Version $Rev: 167 $
 * @Author M.Sekido
 * @Date  $Date: 2015-09-13 12:33:13 +0900 (日, 13  9月 2015) $
 * $Id: ExtSubCh.hpp 167 2015-09-13 03:33:13Z vlbi $ 
 */

#include "DAS_info.hpp"
#include <vector>
#include <string>
#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cmath>
// using namespace std;

extern "C"{
#include <stdint.h>
}

class ExtSubCh {
public:
  enum mode_t{Mk4GeoVSI, Normal};
  enum BitAsgn_t {MSB1of2, LSB1of2};
private:
  const uint8_t* _rawarr;
  const uint8_t* _ptr;
  int32_t _arr_size; ///< array size (byte)
  const uint8_t* _ptr_end;
  int32_t _remain;
  vector <uint8_t>::iterator _iptr;
  const DAS_Info* _indas;
  DAS_Info _outdas;
  BitAsgn_t _MLSB_flag;
  uint8_t _transTbl[256]; ///< Table for 1bit extraction 
  /// Conversion Table generation for 2bit-> 1bit with MSB
  void TableGen_LSB();
  /// Conversion Table generation for 2bit-> 1bit with MSB
  void TableGen_MSB();
  mode_t  mode_type; 
public:
  /// Constructor 
  ExtSubCh();
  ExtSubCh(const uint8_t* a, const int32_t size, const DAS_Info* idas, 
	   const DAS_Info& odas ):
    _rawarr(a), _ptr(a), _arr_size(size), _ptr_end(a+_arr_size),
    _remain(size), _indas(idas), _outdas(odas), mode_type(Normal){; }  
  void Setup(const uint8_t* a, const int32_t size, const DAS_Info* idas, 
	     const DAS_Info& odas, BitAsgn_t bit_asign = MSB1of2 );
  /// Set Output DAS-Information 
  void Set_OutDas(const DAS_Info& das){_outdas = das;}
//   /*! Extracting data to *ptr by <size> amount 
//    * @param *ptr Point for output extracted data.
//    * @param size Data Size to be extracted.
//    * @return  
//    *   - -1: Minor error, to be try again
//    *   - -2: Serious error, to be returned without trying again.
//    */ 
//   int Extract(uint8_t* ptr, const int32_t size);
  /*! Extracting data to ptr by size amount under keeping Quantization nbit constant. 
   * Thus convertion from nbit->mbit is out of scope of this function.
   * @param ptr Reference to vector array <uint8_t> for extracted data output.
   * @param size Data Size to be retuned.
   * @return  
   *   - -1: Minor error, to be try again
   *   - -2: Serious error, to be returned without trying again.
   */ 
  int Extract(vector <uint8_t>& ptr, const int32_t size);
  /**! Extracting data with converting 2bit -> 1bit quantization.
   * \param prt Pointer for output 
   * \param size Nubmer of bytes requested.
   * \return  
   *   - -1: Minor error, to be try again
   *   - -2: Serious error, to be returned without trying again.
   */
  int Extract1bit_from2bit(vector <uint8_t>& ptr, const int32_t size);
  void Set_Mode(const mode_t a){mode_type = a;}
private:
  /** Extracting Subchannels, pointer version
   * \return Number of bytes 
   */
  int Ext32_2( uint8_t* arr, const int32_t rqbyte);
  int Ext32_4( uint8_t* arr, const int32_t rqbyte);
  int Ext32_8( uint8_t* arr, const int32_t rqbyte);
  int Ext32_16( uint8_t* arr,const int32_t rqbyte);
  int Ext32_2Mk4geo( uint8_t* arr, const int32_t rqbyte);
  int Ext32_4Mk4geo( uint8_t* arr, const int32_t rqbyte);
  int Ext32_8Mk4geo( uint8_t* arr, const int32_t rqbyte);
  int Ext32_16Mk4geo( uint8_t* arr,const int32_t rqbyte);
  int Ext32_32Mk4geo( uint8_t* arr,const int32_t rqbyte);
  int Ext32_2to1bit2( uint8_t* arr, const int32_t rqbyte);
  int Ext32_2to1bit4( uint8_t* arr, const int32_t rqbyte);
  int Ext32_2to1bit8( uint8_t* arr, const int32_t rqbyte);
  int Ext32_2to1bit16( uint8_t* arr,const int32_t rqbyte);
  int Ext32_2to1bit2Mk4geo( uint8_t* arr, const int32_t rqbyte);
  int Ext32_2to1bit4Mk4geo( uint8_t* arr, const int32_t rqbyte);
  int Ext32_2to1bit8Mk4geo( uint8_t* arr, const int32_t rqbyte);
  int Ext32_2to1bit16Mk4geo( uint8_t* arr,const int32_t rqbyte);
  int Ext_identical32( uint8_t* arr, const int32_t rqbyte);
  int Ext_identical16( uint8_t* arr, const int32_t rqbyte);
  int Ext_identical8( uint8_t* arr, const int32_t rqbyte);
  int Ext_identical4( uint8_t* arr, const int32_t rqbyte);
  int Ext_identical2( uint8_t* arr, const int32_t rqbyte);
  int Ext_identical1( uint8_t* arr, const int32_t rqbyte);
  int Ext16_1( uint8_t* arr, const int32_t rqbyte);
  int Ext16_2( uint8_t* arr, const int32_t rqbyte);
  int Ext16_4( uint8_t* arr, const int32_t rqbyte);
  int Ext16_4Mk4geo( uint8_t* arr, const int32_t rqbyte);
  int Ext16_8( uint8_t* arr, const int32_t rqbyte);
  int Ext16_2to1bit2( uint8_t* arr, const int32_t rqbyte);
  int Ext16_2to1bit4( uint8_t* arr, const int32_t rqbyte);
  int Ext16_2to1bit8( uint8_t* arr, const int32_t rqbyte);
  int Ext8_1( uint8_t* arr, const int32_t rqbyte);
  int Ext8_2( uint8_t* arr, const int32_t rqbyte);
  int Ext8_4( uint8_t* arr, const int32_t rqbyte);
  int Ext8_2to1bit2( uint8_t* arr, const int32_t rqbyte);
  int Ext8_2to1bit4( uint8_t* arr, const int32_t rqbyte);
  int Ext4_1( uint8_t* arr, const int32_t rqbyte);
  int Ext4_2( uint8_t* arr, const int32_t rqbyte);
  int Ext4_2to1bit2( uint8_t* arr, const int32_t rqbyte);
  int Ext4_2to1bit1( uint8_t* arr,  const int32_t rqsize);
  int Ext2_1( uint8_t* arr, const int32_t rqbyte);
  int Ext2_2to1bit1( uint8_t* arr,  const int32_t rqsize);

};
 

#endif  // end of __EXTSUBCH_HPP__
