/** 
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (水, 03  7月 2013) $
 * $Id: K5VSSP_Header.cpp 56 2013-07-03 11:40:14Z vlbi $ 
 */

#include "K5VSSP_Header.hpp"

const int  K5VSSP_Hdr::Set_Srate(const int32_t Hz) {
  int ret=0;
  switch(Hz){
  case    40000: sfreq_b =0; break;
  case   100000: sfreq_b =1; break;
  case   200000: sfreq_b =2; break;
  case   500000: sfreq_b =3; break;
  case  1000000: sfreq_b =4; break;
  case  2000000: sfreq_b =5; break;
  case  4000000: sfreq_b =6; break;
  case  8000000: sfreq_b =7; break;
  case 16000000: sfreq_b =8; break;
  case 32000000: sfreq_b =9; break;
  case 64000000: sfreq_b =10; break;
  default:
    cerr << "E K5VSSP32_Hdr::Set_Srate(): Freq (" << Hz 
	 << ") is unexpected.\n";
    sfreq_b =0; 
    ret = -1;
    break;
  }
  return ret;
}

const int  K5VSSP_Hdr::Set_Qbit(const int q) {
  int ret =0;
  switch (q){
  case 1: 
  case 2:
  case 4:
  case 8:
    qbit_b =(int)(log((double)q)/log(2.));
    break;
  default:
    cerr << "E K5VSSP32_Hdr::Set_Qbit(): qbit invalid(" << q
	 << ") but must be 1,2,4,8." << endl;
    ret = -1;
    break;
  }
  return ret;
}

const int  K5VSSP_Hdr::Set_Nch(const int n) {
  switch(n){
  case 1:nch_b=0;break;
  case 4:nch_b=1;break;
  default:
    cerr << "E K5VSSP32_Hdr::Set_Nch(): nch invalid(" << n
	 << ") but must be 1 or 4." << endl;
    return -1;
  }
  return 0;
}

// ######## K5VSSP32 #### From Here 
const int  K5VSSP32_Hdr::Set_Srate(const int32_t Hz) {
  int ret =0;
  switch(Hz){
  case    40000: sfreq_b =0; break;
  case   100000: sfreq_b =1; break;
  case   200000: sfreq_b =2; break;
  case   500000: sfreq_b =3; break;
  case  1000000: sfreq_b =4; break;
  case  2000000: sfreq_b =5; break;
  case  4000000: sfreq_b =6; break;
  case  8000000: sfreq_b =7; break;
  case 16000000: sfreq_b =8; break;
  case 32000000: sfreq_b =9; break;
  case 64000000: sfreq_b =10; break;
  default:
    cerr << "E K5VSSP32_Hdr::Set_Srate(): Freq (" << Hz 
	 << ") is unexpected.\n";
    sfreq_b =0; break;
    ret = -1;
  }
  return ret;
}

const int K5VSSP32_Hdr::Set_Qbit(const int q) {
  int ret=0;
  switch (q){
  case 1: 
  case 2:
  case 4:
  case 8:
    qbit_b =(int)(log((double)q)/log(2.));
    break;
  default:
    cerr << "E K5VSSP32_Hdr::Set_Qbit(): qbit invalid(" << q
	 << ") but must be 1,2,4,8." << endl;
    ret = -1;
    break;
  }
  return ret;
}

const int  K5VSSP32_Hdr::Set_Nch(const int n) {
  switch(n){
  case 1:nch_b=0;break;
  case 4:nch_b=1;break;
  default:
    cerr << "E K5VSSP32_Hdr::Set_Nch(): nch invalid(" << n
	 << ") but must be 1 or 4." << endl;
    return -1;
  }
  return 0;
}

//======= VSSP32 libraries ==== from here ======
void  K5VSSP32_Hdr::Disp() {
  fprintf(stderr,"sync=%0X sync2=%x \n",sync_word, sync2);
  fprintf(stderr,"Epoch:%04d/%03d sod=%0u \n",year,doy,sod);
  fprintf(stderr,"nch=%x sfreq=%x qbit=%x \n",nch_b, sfreq_b, qbit_b);
  fprintf(stderr,"ver=%d.%d size=%lu\n",ver_mj, ver_mn, sizeof(K5VSSP_Hdr));
  Print_Aux_Info();

  uint32_t *pt=(uint32_t*) this;
  for(int i=0; i<8; i++){ fprintf(stderr, "c(hex)= %0X ", *pt++); }
  putc('\n',stderr);
}

void K5VSSP32_Hdr::Print_Aux_Info(FILE *fp){

  K5_Aux_1 *p =(K5_Aux_1*)ext_aux;
  fprintf(fp, "\t AuxFmt= %d Fltr= %d MHz", p->aux_fmt_No, p->fltr_freq );

  switch(p->aux_fmt_No ){
  case 2:{
    K5_Aux_2 *paux_2 = (K5_Aux_2*)ext_aux; 
    char pcname[80]; strncpy(pcname,paux_2->hostPCname,8); pcname[9]='\0';
    fprintf(fp," Host= %8s", pcname);
  } break;
  case 1:
    {
      char pcname[80]; 
      strncpy(pcname,p->hostPCname,8);pcname[8]='\0';
      char stationname[80]; 
      strncpy(stationname,p->stname,8);stationname[8]='\0';
      char stid[80]; 
      strncpy(stid,p->station_id,2);stid[2]='\0';
      fprintf(fp," Host= %8s Stid: %2s Stname: %8s", 
	      pcname, stid, stationname );
    }  break;
  }
  
}

K5VSSP32_Hdr*   K5VSSP32_Hdr_find(const uint8_t*  buf, 
				  const uint32_t nbyte, int& offset){
  int32_t remain = nbyte;
  int find_flag=0; 
  uint8_t *pb = (uint8_t* )buf;
  K5VSSP32_Hdr *pt_head(0);
  int32_t count=0;

  int32_t hdsize = pt_head->Headsize();
  //  int32_t hdsize = aaa();
  while( remain >= hdsize
	 && find_flag==0){
    pt_head  = (K5VSSP32_Hdr*)pb;
    //    if(dbg_disp_level>10) pt_head->Disp();
    if( pt_head->sync_word == 0xffffffff &&
	(pt_head->sync2 == 0x8b || pt_head->sync2 == 0x8c)){
      find_flag =1;
      //      memcpy(&head_buf[0], pb, Headsize());
      //      pt_head = (K5VSSP_Hdr*)&head_buf[0];
      offset=count;
      return(pt_head);
    }else{
      count++; remain--;
    }
  } // end while 
  return(0);
}

K5VSSP32_Hdr*   K5VSSP32_Hdr_find(FILE *fp, int& offset){

  //  int find_flag=0; 

  K5VSSP32_Hdr *pt_head(0);
  int32_t count=0;
  int32_t hdsize = pt_head->Headsize();
  int bufsize = hdsize * 10;
  uint8_t buf[bufsize];

  //  int32_t hdsize = aaa();
  while( feof(fp) ==0 ){
    int nread = fread(buf,bufsize,1,fp);
    if( nread <1) break;
    pt_head  = (K5VSSP32_Hdr*)buf;

    for(int i=0; i<bufsize; i++){
      //    if(dbg_disp_level>10) pt_head->Disp();
      if(  pt_head->sync_word == 0xffffffff &&
	  (pt_head->sync2 == 0x8b || pt_head->sync2 == 0x8c)){
	offset=count;
	fseek(fp,i-bufsize,SEEK_CUR);
	return(pt_head);
      }else{
	count++; 
      }
    }
  } // end while 
  return(0);
}
  
Time_dat K5VSSP32_Hdr::Epoch(){
  Time_dat t1;
  double soy = (Doy()-1)*86400. + Sod();
  t1.Set_time(Year(),soy);
  return t1;
}


void K5VSSP_Hdr::Print_hd(FILE *fp){
  fprintf(fp, "- K5VSSP HD info:");
  fprintf(fp," Epoch: sod=%d", sod );
  //    fprintf(fp," Sod: %d", pt_head->Sod());
  fprintf(fp, " Nch= %d Sampl= %u Bit= %d\n", 
	  Nch(), Sfreq(), Qbit() );
}

void K5VSSP32_Hdr::Print_hd(FILE *fp){
  fprintf(fp, "- K5VSSP32 HD info:");
  fprintf(fp," Epoch= %s", Epoch().Print_time(20).c_str());
  fprintf(fp, " Nch= %d Sampl= %u Bit= %d", 
	  Nch(), Sfreq(), Qbit() );
  fprintf(fp," Ver: %d.%d \n", ver_mj, ver_mn);
  Print_Aux_Info(fp);
    
  putc('\n',fp);

}


//======= VSSP32 libraries ==== to here ======
