/** 
 * @Version $Rev: 154 $
 * @Author M.Sekido
 * @Date  $Date: 2014-11-25 13:11:37 +0900 (火, 25 11月 2014) $
 * $Id: VDIF_Frame.cpp 154 2014-11-25 04:11:37Z vlbi $ 
 */


#include <cstring>
#include "VDIF_Frame.hpp"
extern "C"{
#include <errno.h>
#include <arpa/inet.h>
}

VDIF_Frame::VDIF_Frame():VLBI_Frame(DEFAULT_MTU_SIZE),_fp(0),
			 _workbuf(0),_hdr_tested(-1)  //nframe_sec(0) del 2014/5/20 miya
{
    Time_dat t1(2000,1,1,0,0,0);
    //    _ref_epoch0 = t1;
    _vdif_hdr.Set_Nch(das_inf.Nch());
    _vdif_hdr.Set_Qbit(das_inf.Qbit());
    _vdif_hdr.Set_Srate(das_inf.Srate()/1000);
}


const long VDIF_Frame::Read_every_sec_hdr(vector <VDIF_Hdr>& hdrarr,
					  const size_t nsec, const Read_t mode)
{
  if(_fp==0){
    cerr << "E Failed VDIF_Frame::Test_Read(): File is not opened yet\n";
    return -1;
  }

  long offset(0); /// First reading Header
  if(! VDIF_Hdr_find(_fp, offset, &_vdif_hdr)){
    cerr << "Error: Failed to find VDIF-Header \n";
    return -1;
  }; 

  _hdr_tested = 0; ///< Set test-flag to 'OK'
  hdrarr.resize(0);
  framesize = _vdif_hdr.Framesize();
  char *dummy = new char[framesize];
  VDIF_Hdr* phdr = (VDIF_Hdr*)dummy;

  unsigned long sec_dat[2]; // = {vdif_hdr.Sec_Ref(),0}; // the first.
  sec_dat[0] = _vdif_hdr.Sec_Ref();
  sec_dat[1] = 0;
  hdrarr.push_back(_vdif_hdr);

  // Search for the next 1sec
  int sec_count(0);
  long frm_sec_cnt=-1;
  long frm_sec0=0;
  //  long frm_intvl(0);
  long frm_count(0);
  nframe_sec =0;
  while( nsec == 0 || hdrarr.size() < nsec ){
    if(!VDIF_Hdr_find(_fp, offset, phdr)) break;
    if(fseek(_fp,framesize,SEEK_CUR) <0) break;
    if(offset != 0) _hdr_tested++;
    frm_count++;
    frm_sec_cnt++;
    if( (sec_dat[1] = phdr->Sec_Ref()) != sec_dat[0]){
      hdrarr.push_back(*phdr);
//       if(sec_dat[1] - sec_dat[0] == 1)
// 	frm_intvl = hdrarr[sec_count].iFrame() - phdr->iFrame();
      sec_dat[0] = sec_dat[1];
      sec_count++;    
      
      if(frm_sec0 == 0) // First 
	frm_sec0 = frm_sec_cnt; 
      else {  // second and later
	if(nframe_sec != 0 || frm_sec0 == frm_sec_cnt){
	  nframe_sec = frm_sec0;
	  if(mode == QUICK) {
	    fseek(_fp,framesize*(nframe_sec - 1),SEEK_CUR); // Skip 1sec
	    frm_count += nframe_sec - 1;
	  }
	}else
	  frm_sec0 = frm_sec_cnt; 
      } // else
      frm_sec_cnt=0;
    };  // if
  } // while
  //  if(frm_intvl >0) nframe_sec = frm_intvl;
  //  nframe_sec = frm_count/sec_count;

  int nch = _vdif_hdr.Nch();
  int qbit = _vdif_hdr.Qbit();
  uint32_t srate(0);
  switch (hdrarr[0].EDV()){
  case EDV_NICT:
  case EDV_NRAO:
    srate = _vdif_hdr.Srate();
    break;
  case EDV_HAYSTACK:
  default:
    // Set Srate for reference for non-NICT version
    if( frm_count>0){ 
      long datsize = Framesize() - Headsize();
      srate = frm_sec0 * datsize * 8/qbit/nch;
      //      uint32_t srate_ksps = frm_sec0 * datsize * 8/hdrarr.size()/qbit/nch/1000;
      // vdif_hdr.Set_Srate(srate_ksps);
      // for(int i=0; i< (int)hdrarr.size(); i++) hdrarr[i].Set_Srate(srate_ksps);
    }
    break;
  }

  // Set DAS information for latter use of Srate
  Set_Sparam(nch,qbit,srate );
    
  delete [] dummy;
  return frm_count;
  
}

const long VDIF_Frame::Read_every_hdr(vector <VDIF_Hdr>& hdrarr )
{
  if(_fp==0){
    cerr << "E Failed VDIF_Frame::Test_Read(): File is not opened yet\n";
    return -1;
  }

  long offset(0); /// First reading Header
  if(! VDIF_Hdr_find(_fp, offset, &_vdif_hdr)){
    cerr << "Error: Failed to find VDIF-Header \n";
    return -1;
  }; 

  _hdr_tested = 0; ///< Set test-flag to 'OK'
  hdrarr.resize(0);
  framesize = _vdif_hdr.Framesize();
  char *dummy = new char[framesize];
  VDIF_Hdr* phdr = (VDIF_Hdr*)dummy;
  fseek(_fp,framesize,SEEK_CUR);
  
  // unsigned long sec_dat[2]; // = {vdif_hdr.Sec_Ref(),0}; // the first.
  // sec_dat[0]=0;
  // sec_dat[1]=vdif_hdr.Sec_Ref();
  hdrarr.push_back( _vdif_hdr);
  
  // Search for the next 1sec
  long frm_sec0=0;
  //  long frm_intvl(0);
  long frm_count(0);
  while( ! feof(_fp) ){
    if(!VDIF_Hdr_find(_fp, offset, phdr)) break;
    if(fseek(_fp,framesize,SEEK_CUR) <0) break;
    if(offset != 0) _hdr_tested++;
    hdrarr.push_back(*phdr);
    if( phdr->iFrame() == 0 ){
      nframe_sec = frm_count - frm_sec0 ;
      frm_sec0 = frm_count;
    }
    
    frm_count++;
  } 


  int nch = _vdif_hdr.Nch();
  int qbit = _vdif_hdr.Qbit();
  uint32_t srate(0);
  switch (hdrarr[0].EDV()){
  case EDV_NICT:
  case EDV_NRAO:
    srate = _vdif_hdr.Srate();
    break;
  case EDV_HAYSTACK:
  default:
    // Set Srate for reference for non-NICT version
    if( frm_count>0){ 
      long datsize = Framesize() - Headsize();
      srate = frm_sec0 * datsize * 8/hdrarr.size()/qbit/nch;
    }
    break;
  }

  // Set DAS information for latter use of Srate
  Set_Sparam(nch,qbit,srate );

  // // Set Srate for reference for non-NICT version
  // switch (hdrarr[0].EDV()){
  // case  EDV_NICT:
    
  //   && frm_count>0){ 
  //   long datsize = Framesize() - Headsize();
  //   int nch = hdrarr[0].Nch();
  //   int qbit = hdrarr[0].Qbit();
  //   uint32_t srate_ksps = nframe_sec * datsize * 8/qbit/nch/1000;
  //   _vdif_hdr.Set_Srate(srate_ksps);
  //   for(int i=0; i< (int)hdrarr.size(); i++) hdrarr[i].Set_Srate(srate_ksps);
  // }

  // // Set DAS information for latter use of Srate
  // Set_Sparam(vdif_hdr.Nch(), vdif_hdr.Qbit(), vdif_hdr.Srate() );

  delete [] dummy;
  return frm_count;
  
}

const long VDIF_Frame::Read_sec( vector <uint8_t>& rawbuf, const int sec){
  vector <VDIF_Hdr> hdrarr;
  if( Tested() < 1 ||  Read_every_sec_hdr(hdrarr, sec) <=0){
      cerr << "  E Failed to VIDF_Frame::Read_sec()" << endl;
      return -1;
  };

  long pktsize = hdrarr[0].Framesize();
  size_t bufsize = nframe_sec* pktsize * sec;
  if(rawbuf.size() != bufsize)  rawbuf.resize(bufsize);
  long cnt = fread( &rawbuf[0], bufsize, 1, _fp);

  long nframe = cnt*bufsize/pktsize;
  return nframe;
}

const ReturnFrameInfo VDIF_Frame::Read_frame( vector <VDIF_Hdr>& hdrarr, 
				   vector <uint8_t>& datbuf,
				   const int nframe)
{
  if( _workbuf.size() < 64) _workbuf.resize(64);
  // Set Pointers 
  VDIF_Hdr *phdr =  (VDIF_Hdr*)&_workbuf[0];
  size_t hdsize = phdr->Headsize();
  
  ReturnFrameInfo ret_frameinfo;
  // Test reading frist buffer
  if(  _workbuf.size() < hdsize){
    _workbuf.resize(hdsize);
    phdr =  (VDIF_Hdr*)&_workbuf[0];
  }
  long cnt = fread( &_workbuf[0], hdsize, 1, _fp);

  if(cnt < 1){
    perror("E VDIF_Frame::Read_frame():");
    char buf[1024];
    string msg = "E VDIF_Frame::Read_frame():";
    msg += strerror_r(errno,buf,1024);
    ret_frameinfo.Set_NG(msg);
    return ret_frameinfo;
  }
  fseek(_fp,-hdsize,SEEK_CUR); // Rewind back

  long framesize = phdr->Framesize(); 
  size_t bufsize = framesize * nframe;
  long datsize = framesize - hdsize;
  uint32_t nfrm_sec = Nframe_sec();
  _vdif_hdr = *phdr; // Set first data to the common hdr-data in the frame

  if(  _workbuf.size() < bufsize){
    _workbuf.resize(bufsize); 
    phdr =  (VDIF_Hdr*)&_workbuf[0];
  }
  
  
  size_t datbufsize = datsize * nframe;  
  if(  datbuf.size() != datbufsize)  datbuf.resize(datbufsize);
  if(  hdrarr.size() != (size_t)nframe)  hdrarr.resize(nframe);
  
  uint8_t *ptr = &_workbuf[0] + hdsize;
  uint8_t *ptrd0 = &datbuf[0];
  uint8_t *ptrd = ptrd0;

  int32_t iframe0 = _vdif_hdr.iFrame();
  int32_t iframe_nxt = (iframe0+1) % nfrm_sec;
  uint frmcount=0; 
  uint32_t sec1 = _vdif_hdr.Sec_Ref();
  for(int i=0; i< (int)nframe; i++){
    if(fread( &_workbuf[0], framesize,1,_fp) < 1){
      perror("E VDIF_Frame::Read_frame():");
      string msg = " VDIF_Frame::Read_frame():EOF found:";
      char buf[1024];
      msg += strerror_r(errno,buf,1024);
      ret_frameinfo.Set_NG(msg);
      break;
    }
    if( i == 0 ||iframe_nxt == phdr->iFrame()){
      hdrarr[i] = *phdr;
      sec1 = phdr->Sec_Ref();
      memcpy(ptrd, ptr, datsize);
      
      ptrd += datsize; 
      iframe_nxt = (phdr->iFrame() +1) % nfrm_sec;
      frmcount++;
      ret_frameinfo.Set_OK(frmcount);
    }else{
      fseek(_fp,-framesize,SEEK_CUR); // rewind by one framesize
      int jump = phdr->iFrame() - iframe_nxt;
      jump += jump < 0 ? nfrm_sec : 0;
      int jsec = phdr->Sec_Ref() - sec1;
      jump += jsec * nfrm_sec;
      ret_frameinfo.Set_Jump(frmcount, jump);
      fprintf(stderr,"W There was a jump of iFrame(): expected %d -> %d\n",
	      iframe_nxt, phdr->iFrame());
      break;
    }
  }

//   double dt = (double)datsize/4/DAS_Info().Speed();
//   Time_dat t1 = vdif_hdr.Epoch() + dt* vdif_hdr.iFrame(); 
//   Set_Epoch(t1);
  
  return ret_frameinfo;
}
  
const uint32_t VDIF_Frame::Nframe_sec(void){

  if(nframe_sec == 0 ){
    int32_t speed = _vdif_hdr.Speed();
    if( speed > 0) nframe_sec = speed*4/_vdif_hdr.Framesize();
  }
  return nframe_sec;
}
  
/** Set frame size by dividing pagesize(1sec) with a integer number */
void VDIF_Frame::Set_Nframe_sec(const uint32_t n){
    int32_t byte_sec = das_inf.Bytes_sec();
    int n1 =n;
    while( byte_sec % (8*n1) !=0 && byte_sec > 8*n1){
	n1++;
    }
    int32_t pld_size = byte_sec/n1;
    framesize = pld_size + VDIF_Hdr_size;
    _vdif_hdr.Set_FrameLen(framesize);
    fprintf(stderr,
	    "Info: VDIF_Frame::Set_FrameDiv(%d) fixed Ndiv=%d, Framesize=%d\n",
	    n,n1,framesize);
    VLBI_Frame::nframe_sec = n1; // add 2014/5/22 miya
}


