/** 
 * @Version $Rev: 148 $
 * @Author M.Sekido
 * @Date  $Date: 2014-10-20 16:40:55 +0900 (Mon, 20 Oct 2014) $
 * $Id: VLBI_Frame.cpp 148 2014-10-20 07:40:55Z vlbi $ 
 */

#include "VLBI_Frame.hpp"


/** Set Sampling parameters, s:(Hz),  return data rate (Bytes/sec)  */
int32_t 
VLBI_Frame::Set_Sparam(const int n, const int q, const uint32_t s){
//   if(das_inf==0){
//     cerr << "Error in VLBI_Frame::Set_Param(): DAS_Inf is not set \n";
//     return -1;
//   }
  das_inf.Set_Param(n,q,s); 
  return das_inf.Speed()*4; 
}
  

/// ################# VSSP32 From here 
/** Set Sampling parameters, s:(Hz),  return data rate (Bytes/sec)  */
int32_t K5VSSP32_Frame::Set_Sparam(const int n, const int q, const uint32_t s){
  VLBI_Frame::Set_Sparam(n,q,s);
  k5hdr.Set_Qbit(q);
  k5hdr.Set_Srate(s);
  if(k5hdr.Set_Nch(n)<0){  return -1;  }
  framesize = k5hdr.Framesize();
  return das_inf.Speed()*4;
}

void K5VSSP32_Frame::Set_Epoch(const Time_dat& t1){
  data_epoch=t1;
  k5hdr.Set_Epoch(t1);

  if(Logsystem::Test_Logsys(Logsystem::DEBUG))  {
    string func_id("K5VSSP32_Frame::Set_Epoch(Time_dat t1):");
    string tmstr = t1.Print_time(20);
    Logsystem::debug_logger()->Tstamp() 
      << "D " << func_id << "t1=(" << tmstr << ")\n";
    Logsystem::debug_logger()->Flush();
  }

}

void K5VSSP32_Frame::Set_Epoch(const time_t t1){
 
  struct tm tm1;
  //  time_t t2 =t1;
  gmtime_r(&t1, &tm1);
  int yy = tm1.tm_year + 1900;
  int mon = tm1.tm_mon + 1;
  int dd = tm1.tm_mday;
  int hh = tm1.tm_hour;
  int mm = tm1.tm_min;
  int ss = tm1.tm_sec;
  Time_dat td(yy,mon,dd,hh,mm,ss);
  Set_Epoch(td);

  if(Logsystem::Test_Logsys(Logsystem::DEBUG))  {
    string func_id("K5VSSP32_Frame::Set_Epoch():");
    string tmstr = td.Print_time(20);
    Logsystem::debug_logger()->Tstamp() 
      << "D " << func_id << ", yy="<< yy
      << "td=(" << tmstr << "), "
      << " VSSP32Hdr.Year=" << k5hdr.Year() 
      << ", Doy=" << k5hdr.Doy() << ", Sod=" << k5hdr.Sod() << "\n";
    Logsystem::debug_logger()->Flush();
  }
}

int  K5VSSP32_Frame::Set_header(const uint8_t* buf, const int ifrm){
  K5_Aux_1* pk5aux = (K5_Aux_1*)k5hdr.ext_aux;
  pk5aux->aux_fmt_No = 1;
  //  k5hdr.Set_Stname(stname);
  //  k5hdr.Set_StnID(id);  
  memcpy(pk5aux->station_id, stid.c_str(),2);
  memcpy(pk5aux->stname,stname.c_str(),8);
  char host[80];
  gethostname(host,80);
  int len = strlen(host);
  len = len > 8 ? 8 :len;
  memcpy(pk5aux->hostPCname,host,len);
  //  memcpy((void*)buf,&k5hdr, sizeof(k5hdr));
  memcpy((void*)buf,&k5hdr, K5VSSP32_Hdr_size);
  return sizeof(k5hdr);
}


const int K5VSSP32_Frame::Set_DasInfo(const DAS_Info& ds){
  das_inf = ds;
  try{
    if( k5hdr.Set_Qbit(ds.Qbit()) < 0 ) throw -1;
    if( k5hdr.Set_Srate(ds.Srate()) < 0 ) throw -2;
    if( k5hdr.Set_Nch(ds.Nch()) <0 ) throw -3;
  }
  catch(const int ret){
    switch(ret){
    case -1:
      cerr << "Err at K5VSSP32_Frame::Set_DasInfo(): Incompatible Qbit("
	   << ds.Qbit() << ") parameters:" << ds.Mode() << endl;
      break;
    case -2:
      cerr << "Err at K5VSSP32_Frame::Set_DasInfo(): Incompatible Srate("
	   << ds.Srate() << ") parameters:" << ds.Mode() << endl;
      break;
    case -3:
      cerr << "Err at K5VSSP32_Frame::Set_DasInfo(): Incompatible Nch("
	   << ds.Nch() << ") parameters:" << ds.Mode() << endl;
      break;
    default:
      cerr << "Error at K5VSSP32_Frame::Set_DasInfo(): Incompatible DAS parameters:"  << ds.Mode() << endl;
      
      return -1;
      break;
    }
  }

  catch(const char* msg){
    cerr << "Error at K5VSSP32_Frame::Set_DasInfo(): Incompatible "
	 << msg << " parameters:"  << ds.Mode() << endl;
    return -1;
  }
  framesize = k5hdr.Framesize();
  return 0;
}

/// ################# VSSP32 Until here 

/// ################# Misclanious tools 
double tm2mjd(struct tm date) {
  int  y, c;
  //  m, x1, x2, x3;
  double dayfrac;

  if (date.tm_mon < 2) {
    //    m = date.tm_mon+10;
    y = date.tm_mon+1900-1;
  } else {
    //    m = date.tm_mon-2;
    y = date.tm_year+1900;
  }

  c = y/100;
  y = y-c*100;

  // x1 = 146097*c/4;
  // x2 = 1461*y/4;
  // x3 = (153*m+2)/5;

  dayfrac = ((date.tm_hour*60.0+date.tm_min)*60.0+date.tm_sec)
    /(60.0*60.0*24.0);

  return(cal2mjd(date.tm_mday, date.tm_mon+1, date.tm_year+1900)
	 +dayfrac);
}


// void VLBI_Frame::Set_Epoch(const time_t t1){
//   if(ref_epoch0.Mjd() ==0) 
//     ref_epoch0 = vdif_hdr.Set_Epoch0(t1);

//   data_epoch = t1; 
//   vdif_hdr.Set_Epoch(t1);
// }


double cal2mjd(int day, int month, int year) {
  int m, y, c, x1, x2, x3;

  if (month <= 2) {
    m = month+9;
    y = year-1;
  } else {
    m = month-3;
    y = year;
  }

  c = y/100;
  y = y-c*100;

  x1 = 146097*c/4;
  x2 = 1461*y/4;
  x3 = (153*m+2)/5;

  return(x1+x2+x3+day-678882);
}

