
#ifndef __BITUTL_HPP__
#define __BITUTL_HPP__

/** Bit Opearation Utilities .
 * @Version $Rev: 131 $
 * @Author M.Sekido
 * @Date  $Date: 2014-07-25 14:12:03 +0900 (Fri, 25 Jul 2014) $
 * $Id: bitutl.hpp 131 2014-07-25 05:12:03Z vlbi $ .
 */

#include <vector>
#include <string>
extern "C" {
#include  "strings.h"
}
#include "DAS_info.hpp"

/** bit counter */
inline const int bitcnt(const uint32_t a){
  
  uint32_t x=a;
  int cnt=0;
  for(int i=0; i<32; i++){
    if(x & 0x01) cnt++;
    x >>= 1;
  }
  return cnt;
}

#ifndef MAX_NCH
#define MAX_NCH (16)
#endif

class Decode{
  /**! Bit encoding order, Same with ADS2000 or its reverse. 
   * BINARY_2bit     -> 00, 01, 10, 11, from bottom to the top level
   * RevBINARY_2bit  -> 00, 10, 01, 11, from bottom to the top level
   * SignMag_2bit    -> 01, 00, 10, 11, from bottom to the top level
   * RevSignMag_2bit -> 10, 00, 01, 11, from bottom to the top level
   */
  static const float level_1bit[2];
  static const float level_BINARY_2bit[4];
  static const float level_RevBINARY_2bit[4];
  static const float level_SignMag_2bit[4];
  static const float level_RevSignMag_2bit[4];
  DAS_Info *dasInfo;
  vector <float> fvalarr[16]; 
  float  ftable[256][MAX_NCH];
  
public:
  enum BitEnc_t {BINARY_2bit, RevBINARY_2bit, SignMag_2bit, RevSgnMag_2bit};
  /** Constructor */
  Decode(DAS_Info *dasinf):dasInfo(dasinf){;}    
  /** Making look-up table. */
  const int Make_Table(const BitEnc_t enc_type=BINARY_2bit);
  /** Interpretation (decoding)
   * \return Number of data points.   */
  const long Interpret( const long npt=0);
  /** Interpretation (decoding) 
   * \return Number of data points.   */
  const long Interpret(vector <uint8_t>& datbuf, const long npt);
  /** Returns reference of value array at ch*/
  vector <float>& fValArr(const int ch){ return fvalarr[ch];}
  const float *pfValArr(const int ch){ return &fvalarr[ch][0];}
private:
  const int MkTbl1bit(void);
  const int MkTbl2bit(const BitEnc_t t=BINARY_2bit);
  const int MkTbl4bit(void);
  const int MkTbl8bit(void);
};


inline const long Decode::Interpret(vector <uint8_t>& datbuf, 
				    const long ndat){
  if(dasInfo ==0){  
    cerr << "Error @Decode::Val():dasInfo is empty.\n";
    return -1;  
  }
	

  int nch= dasInfo->Nch();
  int bit = dasInfo->Qbit();
  int dunit = nch * bit;
  int ch_byte = 8/bit;
  int unit_byte(0);
  int byte_unit (0);
  char msg[256];
  try{
    if( nch > ch_byte){  // 1 unit takes more than 1 Byte
      if( dunit % 8 != 0){
	sprintf(msg, 
		"Nch(%d)*Qbit(%d) must be multiple of 8 bit or less than 8bit", nch, bit);
	throw msg;
      }
      byte_unit = dunit / 8;
    }else{  // 1 byte include no less than 1 unit
      if( 8 % dunit != 0 ){
	sprintf(msg, "Nch(%d)*Qbit(%d) must be multiple of 8 bit or less than 8bit", nch, bit);
	throw msg;
      }else{
	unit_byte = 8 / dunit;
      }
    }
  } 
  catch(const char* msg1){
    cerr << "Error in Decode::Val():" << msg1 << endl;
    return -1;
  }
  size_t npt = ndat;
  if(ndat == 0) npt = dasInfo->Srate();
  for( int i=0; i<nch; i++) 
    if( fvalarr[i].size() < npt){
      //      valarr[i].resize(npt);
      fvalarr[i].resize(npt);
    }
  vector <uint8_t>::iterator sptr = datbuf.begin();  
  vector <float>::iterator dptr[nch];
  //  vector <int8_t>::iterator dptr2[nch];

  for(int i=0; i<nch; i++){
    dptr[i] = fvalarr[i].begin();  
    //    dptr2[i] = valarr[i].begin();  
  }

  int32_t cnt=0;
  
  if( byte_unit == 0){ // 1 byte contains multiple units
    while(sptr < datbuf.end() && dptr[0] < fvalarr[0].end()){
      for( int i=0; i< unit_byte; i++){
	for(int j=0; j<nch; j++){
	  *dptr[j] = ftable[*sptr][j+i*nch]; 
	  //	  *dptr2[j] = table[*sptr][j+i*nch]; 
	  dptr[j]++;
	  //	  dptr2[j]++;
	}
	cnt++;
      }
      sptr++;
    } 
  }else{ // 1 unit takes multiple bytes
    while(sptr < datbuf.end() && dptr[0] < fvalarr[0].end()){
      int  ich=0;
      for(int i=0; i<byte_unit; i++){ // bytes/unit
	for(int jch=0; jch< ch_byte; jch++,ich++){ // ch/byte
	  *dptr[ich] = ftable[*sptr][jch];  
	  //	  *dptr2[ich] = table[*sptr][jch];  
	  dptr[ich]++; 	  
	  //	  dptr2[ich]++; 
	}
	sptr++;
      }
      cnt++;
    }
  } 
  
  return cnt;
}




class Bit_hist{
  int qbit;  // 2bit default
  vector <unsigned long *> cnt_arr;
  unsigned long count; ///< Total Counter 
  int nrank; // Number of ranks;
public:
  Bit_hist(const int bit=2):qbit(2){ Init(bit);}
  void Init(const int bit=2){
    qbit = bit;
    int narr = 32 / qbit;
    if(cnt_arr.size() > 0 && cnt_arr[0] !=0){ 
      for(int i=0; i< (int)cnt_arr.size(); i++) delete [] cnt_arr[i];
    }
    cnt_arr.resize(narr);
    nrank=1;
    for(int i=0; i<qbit; i++) nrank *= 2;
    for(int i=0; i<narr;i++){
      cnt_arr[i] = new unsigned long [nrank];
      bzero((void*)cnt_arr[i],sizeof(unsigned long)*nrank);      
    }
    count=0;
  }
  void  Count(const uint32_t a);
  std::string StrHist(const int ich);
};


inline std::string Bit_hist::StrHist(const int ich){
  if(ich < -1 || ich >= (int)cnt_arr.size()){
    fprintf(stderr,"Error: ich(%d) is out of range(%lu)\n", ich, cnt_arr.size());
    return "NG";
  }

  string ret;
  char buf[512];

  if(ich == -1){
    for(int j=0; j<nrank; j++){
      sprintf(buf," [%02x] ", j);
      ret += buf;
    }
  }else{
    double ratio[nrank];
    double dcount = (double)count;
    for(int j=0; j<nrank; j++){
      ratio[j] = (double)cnt_arr[ich][j]/dcount * 100.;
      sprintf(buf,"%5.1f ", ratio[j]);
      ret += buf;
    }
  }
  return ret;  

}


inline void Bit_hist::Count(const uint32_t a){
  
  uint32_t mask = 0x03;
  switch(qbit){
  case 1: mask = 0x01; break;
  case 4: mask = 0x0f; break;
  case 8: mask = 0xff; break;
  case 2: 
  default:
    mask = 0x03; break;
  }

  uint16_t val=0;
  for(int i=0; i < (int)cnt_arr.size(); i++){
    val = (a >> (i * qbit)) & mask;
    cnt_arr[i][val] ++;
  }
  count ++;
}

#endif
