
#ifndef __CRC_HPP__
#define __CRC_HPP__
/** CRC code class 
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (水, 03  7月 2013) $
 * $Id: crc.hpp 56 2013-07-03 11:40:14Z vlbi $ 
 */

#include <cstring>
extern "C" {
#include <stdint.h>
}

const uint16_t  CRC_16_KEY = 0x8005; 
// const uint16_t  CRC_CCITT_KEY =  0x11021;

class CRC_mdl{
protected:
  uint16_t crc_key;
  const char *model_name;
  uint8_t bytetable[256];
public: 
  CRC_mdl(const char* nm):crc_key(CRC_16_KEY) {Init(); model_name=nm;}
  virtual ~CRC_mdl(){;}
  void Set_key(const uint16_t k){crc_key = k;}
  const uint16_t CRC_key()const{ return crc_key;}
  virtual const uint16_t Cmp_CRC(const uint8_t *data, const int nbyte)=0;
  virtual void Init(){init_bitreversal();}
  void init_bitreversal();  
  const uint16_t reversebits16 (const uint16_t s);
  const char* Mdl_Name() const{ return model_name;}
};

class CRC_k5vsi:public CRC_mdl{
public: 
  CRC_k5vsi():CRC_mdl("CRC_K5VSI"){;}
  void Init(){;}
  const uint16_t Cmp_CRC(const uint8_t *data, const int nbyte);
};


const int LBA_CRC_MASK = 040003;
const int LBA_CRC_CYCLE = 16;

class CRC_lba:public CRC_mdl{
  int mask;
  int cycl;
public: 
  CRC_lba():CRC_mdl("CRC_LBA"),
	    mask(LBA_CRC_MASK),cycl(LBA_CRC_CYCLE){;}
  void Init(){;}
  const uint16_t Cmp_CRC(const uint8_t *data, const int nbyte);
};

class CRC_jive:public CRC_mdl{
  uint16_t crc_table[256];
  uint16_t crc_register;
  void initialize_crc_table();
  void put_crc_byte(const uint8_t byte);
  void Reset_crc(){crc_register=0;}
public: 
  CRC_jive():CRC_mdl("CRC_JIVE"),crc_register(0){
    memset(crc_table,0,256*sizeof(uint16_t)); Init();}
  void Init(){initialize_crc_table(); }
  const uint16_t Cmp_CRC(const uint8_t *data, const int nbyte);
};

#endif
