/** 
 * @Version $Rev: 105 $
 * @Author M.Sekido
 * @Date  $Date: 2014-05-21 11:27:01 +0900 (Wed, 21 May 2014) $
 * $Id: time_lib.cpp 105 2014-05-21 02:27:01Z vlbi $ 
 */

#include  "time_lib.hpp"

/*** Month input is 1-12 but counted from 0 to 11  ***/

Mutex Tm_mem::mutex;

double Time_dat::Doy_cal(void){
  int leap=0, doyt[12] = {0,31,59,90,120,151,181,212,243,273,304,334};
  int yy = ymdhms[0];
  int mm = ymdhms[1];
  int dd = ymdhms[2];
  
  if( yy < 100)
    yy += yy < 50 ? 2000 : 1900;
  if( yy % 4 ==0 || (yy % 100 ==0 && yy % 400 != 0)) leap =1;
  
  doy = ydhms[1] = doyt[mm-1] + dd + ( mm >2 ? leap : 0 );

  return(doy);

}

Time_dat::Time_dat(const int yy, const int doy, const int hh, 
		   const int mm, const double ss){ 
  ymdhms[0] = ydhms[0] = yy;
  this->doy = ydhms[1] = doy;
  ydhms[2] = ymdhms[3] = hh;
  ydhms[3] = ymdhms[4] = mm;
  ydhms[4] = ymdhms[5] = (int)rint(ss);
  sec = ss;
  Doy2md();
  Mjd_cal();
  sod = hh * 3600. + mm * 60. + ss;
  if( doy >= 1) soy=(doy-1) *86400 + sod;
  else std::cerr << "Error: doy < 1" << std::endl;

}

Time_dat::Time_dat(int yy, int mon, int dd, int hh, int mm, double ss){ 
  ymdhms[0] = ydhms[0] = yy;
  ymdhms[1] =  mon;
  ymdhms[2] =  dd;
  ydhms[2] = ymdhms[3] = hh;
  ydhms[3] = ymdhms[4] = mm;
  ydhms[4] = ymdhms[5] = (int)ss;
  sec = ss;
  Doy_cal();
  Mjd_cal();
  sod = hh * 3600. + mm * 60. + ss;
  if( doy >= 1) soy=(doy-1)*86400 + sod;
  else  std::cerr << "Error: doy < 1" << std::endl;

}

Time_dat::Time_dat(const double mjd_in){ 
  int yy = 2000;
  Time_dat t_tmp(yy,1,1,0,0,0);
  
  if(mjd_in > t_tmp.Mjd() ){
    while(mjd_in >= t_tmp.Mjd())
      t_tmp.Set_time(++yy,1,1,0,0,0);
    t_tmp.Set_time(--yy,1,1,0,0,0);
  }else{
    while(mjd_in < t_tmp.Mjd())
      t_tmp.Set_time(--yy,1,1,0,0,0);
  }
  ymdhms[0] = ydhms[0] = yy;
  double work = mjd_in - t_tmp.Mjd() + 1. + 1.e-11;
  
  doy = ydhms[1] = (int)floor(work);
  work -= ydhms[1];
  sod = work * 86400;
  sod = rint(sod*10000)/10000.;  /* rounded in 0.1msec */
  ymdhms[3] = ydhms[2] = (int)floor(sod / 3600);
  work *= 86400.;
  work = rint(work*10000)/10000.;  /* rounded in 0.1msec */
  work -= ydhms[2] * 3600.;
  ymdhms[4] = ydhms[3] = (int)floor(work/60);
  work -= ydhms[3] * 60.;
  sec = work;
  ymdhms[5] = ydhms[4] = (int)sec;
  Doy2md();
  mjd = mjd_in;
}

int Time_dat::Set_time(const int yy, const int doy, const int hh, 
			const int mm, const double ss){ 
  int uru = ((yy % 4== 0 && yy % 100 != 0) || yy % 400 == 0) ? 1 : 0;
  if(doy > 365+uru || doy < 1){   err_msg(0, doy);   return -1;  }
  if( hh < 0 || hh > 24){    err_msg(3, hh);    return -1;  }
  if( mm < 0 || mm > 60){    err_msg(4, mm);    return -1;  }
  if( ss < 0 || ss > 60){    err_msg(5, ss);    return -1;  }

  ymdhms[0] = ydhms[0] = yy;
  this->doy = ydhms[1] = doy;
  ydhms[2] = ymdhms[3] = hh;
  ydhms[3] = ymdhms[4] = mm;
  ydhms[4] = ymdhms[5] = (int)ss;
  sec = ss;
  Doy2md();
  Mjd_cal();
  sod =  hh * 3600. + mm * 60. + ss;
  if(doy >= 1) 
    soy=(doy-1)*86400 + sod;
  else{
    std::cerr << "Error: doy < 1" << std::endl;
    return -1;
  }
  return 0;
}

int Time_dat::Set_time(int yy, int mon, int dd, int hh, int mm, double ss){ 
  if(mon > 12 || mon < 1){   err_msg(1, mon);   return -1;  }
  if( dd < 0 || dd > 31){    err_msg(2, dd);   return -1;  }
  if( hh < 0 || hh > 24){    err_msg(3, hh);    return -1;  }
  if( mm < 0 || mm > 60){    err_msg(4, mm);    return -1;  }
  if( ss < 0 || ss > 60){    err_msg(5, ss);    return -1;  }
  
  ymdhms[0] = ydhms[0] = yy;
  ymdhms[1] =  mon;
  ymdhms[2] =  dd;
  ydhms[2] = ymdhms[3] = hh;
  ydhms[3] = ymdhms[4] = mm;
  ydhms[4] = ymdhms[5] = (int)ss;
  sec = ss;
  Doy_cal();
  Mjd_cal();
  sod = hh * 3600. + mm * 60. + ss;
  if( doy >= 1){
    soy=(doy-1)*86400 + sod;
  }else{
    std::cerr << "Error: doy < 1 " << std::endl;
    return -1;
  }
  return 0;
}

int  Time_dat::Set_time(const double mjd_in){ 
  int yy = 2000;
  
  Time_dat t_tmp(yy,1,1,0,0,0);
  
  if(mjd_in > t_tmp.Mjd() ){
    while(mjd_in >= t_tmp.Mjd())
      t_tmp.Set_time(++yy,1,1,0,0,0);
    t_tmp.Set_time(--yy,1,1,0,0,0);
  }else{
    while(mjd_in < t_tmp.Mjd())
      t_tmp.Set_time(--yy,1,1,0,0,0);
  }
  ymdhms[0] = ydhms[0] = yy;
  double work = mjd_in - t_tmp.Mjd() + 1. + 1.e-21;
  
  ydhms[1] = (int)floor(work);
  work -= ydhms[1];
  sod = work * 86400;
  sod = rint(sod*1.e9)/1.e9;  /* rounded in 1nsec */
  ymdhms[3] = ydhms[2] = (int)floor(sod / 3600);
  work *= 86400.;
  work -= ydhms[2] * 3600.;
  work = rint(work*10000)/10000.;  /* rounded in 0.1msec */
  ymdhms[4] = ydhms[3] = (int)floor(work/60);
  work -= ydhms[3] * 60.;
  sec = work;
  ymdhms[5] = ydhms[4] = (int)sec;
  if( fabs(sec)<1.e-3)  sec =0.;
  doy = ydhms[1];
  Doy2md();
  if( doy >= 1){
    soy = (doy-1)*86400.+sod;
  }else{
    std::cerr << "Error: doy < 1 " << std::endl;
    return -1;
  }
  mjd = mjd_in;
  return 0;
}

double  Mjd_cal(int yy, int mm, int dd, int h, int m, double sec)
{  
  double iptr ;
  double mjd;
  
  if( mm <= 2){
    yy--;
    mm += 12;
  }
  int i = (int)(yy/100);
  int k = 2 - i + (int)(i/4);
  mjd = (365.25*yy - modf(365.25*yy, &iptr)) - 679006.;
  mjd+= floor(30.6001*(mm+1))+dd+k;
  mjd += ((sec/60. + m)/60. + h)/24.;
  return(mjd);
}

double  Time_dat::Mjd_cal(void){  
  double yy, mm, iptr ;
  
  yy = ymdhms[0];
  mm = ymdhms[1];
  if( mm <= 2){
    yy--;
    mm += 12;
  }
  int i = (int)(yy/100);
  int k = 2 - i + (int)(i/4);
  mjd = (365.25*yy - modf(365.25*yy, &iptr)) - 679006.;
  mjd+= floor(30.6001*(mm+1))+ymdhms[2]+k;
  mjd += ((sec/60. + ymdhms[4])/60. + ymdhms[3])/24.;
  return(mjd);
}

/*** Resolving from String to ydhms **/
/* Expected format is yyyy/mm/dd hh:mm:ss */
int Time_dat::Set_time(const char *str, const int nterm, const char *dlm)
{
  int i ;
  char *token;
  char *buf = new char [strlen(str)];
  strcpy(buf, str);

  if( (token = strtok(buf, dlm) ) ==NULL){
    fprintf(stderr, "Error: time tag delimiter is missing in str =%s\n",
	    str);
    return -1;
  }
  ymdhms[0] = ydhms[0] = atoi(token);
  switch (nterm){
  case 6: // yyyy mm dd hh mm ss
    for(i=1; i<nterm-1; i++){
      if((token = strtok(NULL, dlm)) ==NULL){
	fprintf(stderr, "Error: time tag delimiter is missing in line =%d\n",i+1);
	return -1;
	//	exit(EXIT_FAILURE);
      }
      ymdhms[i] = atoi(token);
    } /*Loop for i*/
    if((token = strtok(NULL, dlm)) ==NULL){
      fprintf(stderr, "Error: time tag data tag is missing in line =%d\n",i+1);
      return -1;
      //      exit(EXIT_FAILURE);
    }
    sec = atof(token);
    ymdhms[5] = ydhms[4] = (int)rint(sec);
    for(i=0; i<3; i++) ydhms[2+i] = ymdhms[3+i];
    Doy_cal();
    sod = ydhms[2] * 3600. + ydhms[3] * 60. + sec;
    if( doy >= 1)
      soy = (doy-1)*86400. + sod;
    else{
      std::cerr << "Error: doy < 1 " << std::endl;
      return -1;
    }
    Mjd_cal();
    break;
  case 5: // yyyy ddd hh mm ss
    for(i=1; i<nterm-1; i++){
      if((token = strtok(NULL, dlm)) ==NULL){
	fprintf(stderr, "Error: time tag data tag is missing in line =%d\n",i+1);
	return -1;
	//	exit(EXIT_FAILURE);
      }
      ydhms[i] = atoi(token);
    } /*Loop for i*/
    if((token = strtok(NULL, dlm)) ==NULL){
      fprintf(stderr, "Error: time tag data tag is missing in line =%d\n",
	      i+1);
      return -1;
      //      exit(EXIT_FAILURE);
    }
    sec = atof(token);
    ymdhms[5] = ydhms[4] = (int)rint(sec);
    for(i=0; i<3; i++) ymdhms[3+i] = ydhms[2+i];
    doy = ydhms[1];
    sod = ydhms[2] * 3600. + ydhms[3] * 60. + sec; 
    if( doy >= 1) soy = (doy-1) *86400. + sod;
    else{
      std::cerr << "Error: doy < 1 " << std::endl;
      return -1;
    }
    Doy2md();
    Mjd_cal();
    break;
  case 4:
    for(i=1; i<nterm; i++){
      if((token = strtok(NULL, dlm)) ==NULL){
	fprintf(stderr, "Error: time tag data tag is missing in line =%d\n",i+1);
	return -1;
	//	exit(EXIT_FAILURE);
      }
      ydhms[i] = atoi(token);
    } /*Loop for i*/
    sec=0.;
    ymdhms[5] = ydhms[4] = (int)rint(sec);
    for(i=0; i<3; i++) ymdhms[3+i] = ydhms[2+i];
    doy = ydhms[1];
    sod = ydhms[2] * 3600. + ydhms[3] * 60.;
    if( doy >= 1) soy = (doy-1) *86400. + sod;
    else{
      std::cerr << "Error: doy < 1 " << std::endl;
      return -1;
    }
    Doy2md();
    Mjd_cal();
    break;
  default:
    fprintf(stderr,"Error: nterm=%d in Set_time\n", nterm);
    return -1;
    //    exit(EXIT_FAILURE);
  }
  
  delete buf;
  return 0;
}

int  Time_dat::Set_time(const std::string &str, const int nterm, const char *dlm)
{
  std::vector <std::string> token;
  std::string dlmter = dlm;

  //  Tokenize(str, token, dlmter);
  boost::algorithm::split(token, str, boost::is_any_of(dlm));
  
  if( (int)token.size() < nterm){
    std::cerr << "Error: Numbe rof tokes is less than " 
	      << nterm << std::endl;
    std::cerr <<  "  str = " << str << std::endl;
    return -1;
  }
  
  switch (nterm){
  case 6: // yyyy mm dd hh mm ss
    for(int i=0; i<nterm-1; i++)
      ymdhms[i] = atoi(token[i].c_str());
    ydhms[0] = ymdhms[0];
    sec = atof(token[5].c_str());
    ymdhms[5] = ydhms[4] = (int)rint(sec);
    for(int i=0; i<3; i++) ydhms[2+i] = ymdhms[3+i];
    Doy_cal();
    sod = ydhms[2] * 3600. + ydhms[3] * 60. + sec;
    if( doy >= 1) soy = (doy-1)*86400. + sod;
    else{
      std::cerr << "Error: doy < 1 " << std::endl;
      return -1;
    }
    Mjd_cal();
    break;
  case 5: // yyyy doy hh mm ss
    for(int i=0; i<nterm-1; i++)
      ydhms[i] = atoi(token[i].c_str());
    ymdhms[0] = ydhms[0];
    sec = atof(token[4].c_str());
    ymdhms[5] = ydhms[4] = (int)rint(sec);
    for(int i=0; i<3; i++) ymdhms[3+i] = ydhms[2+i];
    doy = ydhms[1];
    sod = ydhms[2] * 3600. + ydhms[3] * 60. + sec; 
    if( doy >= 1) soy = (doy-1) *86400. + sod;
    else{
      std::cerr << "Error: doy < 1 " << std::endl;
      return -1;
    }
    Doy2md();
    Mjd_cal();
    break;
  case 4: // yy doy hh mm ss
    for(int i=0; i<nterm; i++)
      ydhms[i] = atoi(token[i].c_str());
    sec=0.;
    ymdhms[5] = ydhms[4] = 0;
    for(int i=0; i<3; i++) ymdhms[3+i] = ydhms[2+i];
    doy = ydhms[1];
    sod = ydhms[2] * 3600. + ydhms[3] * 60.;
    if( doy >= 1) soy = (doy-1) *86400. + sod;
    else{
      std::cerr << "Error: doy < 1 " << std::endl;
      return -1;
    }
    Doy2md();
    Mjd_cal();
    break;
  default:
    fprintf(stderr,"Error: nterm=%d in Set_time\n", nterm);
    return -1;
  }
  
  return 0;
}


void  Time_dat::Doy2md(void)
{
  int month[12]= {31,59,90,120,151,181,212,243,273,304,334,365}; 
  int i=0, uru=0;


  if(( ydhms[0] % 4 ==0 &&  ydhms[0] % 100 != 0) || ydhms[0] % 400 == 0) uru=1;
  if( month[0] >= doy) ymdhms[1] = 1;
  else{
    i=1;
    while(month[i]+uru < doy) i++;  
    ymdhms[1] = i+1;
  }
  int days=0;
  switch (i){
  case 0:
    days =0;
    break;
  case 1:
    days = month[0] ;
    break;
  default :
    days = month[i-1] +uru ;
    break;
  }
  ymdhms[2] = (int)floor(doy) - days;
}

/*** Resolving from String to ydhms **/
int Time_dat::Set_time(const char str[])
{
  
  int len = strlen(str);
  char timstr[20], *cp;
  int yy, uru;
  
  strcpy(timstr, str);

  switch( len){
  case 11: /* yydddhhmmss*/
    cp = timstr+9;   ymdhms[5] = ydhms[4] = atoi(cp); sec = ymdhms[5]; *cp = '\0';
    cp -=2;  ymdhms[4] = ydhms[3] = atoi(cp); *cp = '\0';
    cp -=2;  ymdhms[3] = ydhms[2] = atoi(cp); *cp = '\0';
    cp -=3;  ydhms[1] = atoi(cp); *cp = '\0';
    ymdhms[0] = ydhms[0] = atoi(timstr);
    Doy2md(); /* Doy 2 mm/dd */
    doy = ydhms[1];
    sod = ydhms[2] * 3600. + ydhms[3] * 60. + ydhms[4];
    if( doy >= 1) soy = (doy-1)*86400. + sod;
    else{
      std::cerr << "Error: doy < 1" << std::endl;
      return -1;
    }
    Mjd_cal();
    break;
  case 12:/* yymmddhhmmss*/
    cp = timstr+10;   sec = ymdhms[5] = ydhms[4] = atoi(cp); sec = ymdhms[5]; *cp = '\0';
    cp -=2;  ymdhms[4] = ydhms[3] = atoi(cp); *cp = '\0';
    cp -=2;  ymdhms[3] = ydhms[2] = atoi(cp); *cp = '\0';
    cp -=2;  ymdhms[2] = atoi(cp); *cp = '\0';
    cp -=2;  ymdhms[1] = atoi(cp); *cp = '\0';
    ymdhms[0] = ydhms[0] = atoi(timstr);
    Doy_cal();
    sod = ydhms[2] * 3600. + ydhms[3] *60. + sec;
    if( doy >= 1) soy = doy*86400. + sod;
    else{
      std::cerr << "Error: doy < 1 " << std::endl;
      return -1;
    }
    Mjd_cal();
    break;
  case 13:/* yyyydddhhmmss*/
    cp = timstr+11; ymdhms[5] = ydhms[4] = atoi(cp); sec = ymdhms[5]; *cp = '\0';
    cp -=2;  ymdhms[4] = ydhms[3] = atoi(cp); *cp = '\0';
    cp -=2;  ymdhms[3] = ydhms[2] = atoi(cp); *cp = '\0';
    cp -=3;  ydhms[1] = atoi(cp); *cp = '\0';
    ymdhms[0] = ydhms[0] = atoi(timstr);
    doy = ydhms[1];
    sod = ydhms[2] * 3600. + ydhms[3] *60. + sec;
    if( doy >= 1) soy = (doy-1)*86400. + sod;
    else{
      std::cerr << "Error: doy < 1" << std::endl;
      return -1;
    }
    yy = ydhms[0];
    if( (yy % 4 ==0 &&  yy % 100 !=0 ) ||  yy % 400 ==0) uru=1; else uru=0;    
    if( 365+uru < ydhms[1] ){
      ydhms[1] -= (356+uru); doy -= 365+uru;
      ydhms[0]++;  ymdhms[0] ++;
    }
    Doy2md();
    Mjd_cal();
    break;
  case 14:/* yyyymmddhhmmss*/
    cp = timstr+12;   ymdhms[5] =  ydhms[4] = atoi(cp); sec = ymdhms[5]; *cp = '\0';
    cp -=2;  ymdhms[4] = ydhms[3] = atoi(cp); *cp = '\0';
    cp -=2;  ymdhms[3] = ydhms[2] = atoi(cp); *cp = '\0';
    cp -=2;  ymdhms[2] = atoi(cp); *cp = '\0';
    cp -=2;  ymdhms[1] = atoi(cp); *cp = '\0';
    ydhms[0] = ymdhms[0] = atoi(timstr);
    yy = ydhms[0];
    if( (yy % 4 ==0 &&  yy % 100 !=0 ) ||  yy % 400 ==0) uru=1; else uru=0;    
    if( 365+uru < ydhms[1] ){
      ydhms[1] -= (356+uru);
      ydhms[0]--;  ymdhms[0] --;
    }
    Doy_cal();
    sod = ydhms[2] * 3600. + ydhms[3] *60. + sec;
    if( doy >= 1) soy = (doy-1)* 86400. + sod;
    else{
      std::cerr << "Error: doy < 1" << std::endl;
      return -1;
    }
    Mjd_cal();
    break;
  default:
    fprintf(stderr, "Error: String (%s) format is wrong. Length = %d\n", str, len);
    return(-1);
    break;
  }

  if(ydhms[0] < 100) ydhms[0] +=  ydhms[0] < 50 ? 2000 : 1900;
  if(ymdhms[0] < 100) ymdhms[0] +=  ymdhms[0] < 50 ? 2000 : 1900;
  
  return(len);

}

/* Compute dhms from sec and returns seconds in double */
void Time_dat::Sec2dhms(double sec)
{
  soy = sec;
  doy = ydhms[1] = (int)floor(sec/86400.)+1;
  sec -= (doy-1) * 86400;
  ymdhms[3] = ydhms[2] = (int)floor(sec / 3600.);
  sec -= ydhms[2]*3600;
  ymdhms[4] = ydhms[3] = (int)floor(sec/60. + 1.e-6);
  sec -= ydhms[3]*60;
  if( sec < 0 && -sec < 1.e-6) sec=0.;
  ymdhms[5] = ydhms[4] = (int)floor(sec);
  this->sec = sec;
  this->sod = sec + (ydhms[2]*60.+ydhms[3])*60.;
}

void Time_dat::Print_time(FILE *fp, const int iptn) const
{
  switch (iptn){
  case 0: // mjd 
    fprintf(fp, "%12.6lf ",mjd);
    break;
  case 10: // yyyy/ddd hh:mm:ss.mmm
    fprintf(fp, "%04d/%03d %02d:%02d:%06.3lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 11: // YYYYyDDDdHHhMMmSS.SSS
    fprintf(fp, "%04dy%03dd%02dh%02dm%06.3lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 12: // YYYY DDD HH MM SS
    fprintf(fp, "%04d %03d %02d %02d %02.0lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 13: // YYYYDDDHHMMSS
    fprintf(fp, "%04d%03d%02d%02d%02.0lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 14: // YYYY/DDD/HH:MM:SS
    fprintf(fp, "%04d/%03d/%02d:%02d:%02.0lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 15: // YYYY/DDD/HH:MM:SS.mmm
    fprintf(fp, "%04d/%03d/%02d:%02d:%06.3lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 107: // yyyy/ddd hh:mm:ss.7777777
    fprintf(fp, "%04d/%03d %02d:%02d:%010.7lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 20: // yyyy/mm/dd hh:mm:ss.mmm 
    fprintf(fp, "%04d/%02d/%02d %02d:%02d:%06.3lf",
	    ymdhms[0],ymdhms[1],ymdhms[2],ymdhms[3],ymdhms[4],sec);
    break;
  case 21: // yyyy/mm/dd hh:mm:ss 
    fprintf(fp, "%04d/%02d/%02d %02d:%02d:%02.0lf",
	    ymdhms[0],ymdhms[1],ymdhms[2],ymdhms[3],ymdhms[4],sec);
    break;
  case 32: // hh mm ss.mmmmmmmm 
    fprintf(fp, "%02d %02d %010.7lf", ymdhms[3],ymdhms[4],sec);
    break;
  default:
    fprintf(fp, "%04d/%02d/%02d (%07.3lf) %02d:%02d:%06.3lf mjd=%.5lf, soy=%.3lf",
	    ymdhms[0],ymdhms[1],ymdhms[2],doy, ymdhms[3],ymdhms[4],sec, mjd, soy);
  }
}

void Time_dat::Print_time(std::string &buf, const int iptn) const
{
  char buf2[256];
  
  switch (iptn){
  case 0: // mjd 
    sprintf(buf2, "%12.6lf",mjd);
    break;
  case 10: // yyyy/ddd hh:mm:ss.mmm
    sprintf(buf2, "%04d/%03d %02d:%02d:%06.3lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 11: // YYYYyDDDdHHhMMmSS.SSS
    sprintf(buf2, "%04dy%03dd%02dh%02dm%06.3lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 12: // YYYY DDD HH MM SS
    sprintf(buf2, "%04d %03d %02d %02d %02.0lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 13: // YYYYDDDHHMMSS
    sprintf(buf2, "%04d%03d%02d%02d%02.0lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 14: // YYYY/DDD/HH:MM:SS
    sprintf(buf2, "%04d/%03d/%02d:%02d:%02.0lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 15: // YYYY/DDD/HH:MM:SS.mmm
    sprintf(buf2, "%04d/%03d/%02d:%02d:%06.3lf",
	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
    break;
  case 20: // yyyy/mm/dd hh:mm:ss.mmm 
    sprintf(buf2, "%04d/%02d/%02d %02d:%02d:%06.3lf",
	    ymdhms[0],ymdhms[1],ymdhms[2],ymdhms[3],ymdhms[4],sec);
    break;
  case 21: // yyyy/mm/dd hh:mm:ss 
    sprintf(buf2, "%04d/%02d/%02d %02d:%02d:%02.0lf",
	    ymdhms[0],ymdhms[1],ymdhms[2],ymdhms[3],ymdhms[4],sec);
    break;
  default:
    sprintf(buf2, "%04d/%02d/%02d (%07.3lf) %02d:%02d:%06.3lf mjd=%.5lf, soy=%.3lf",
	    ymdhms[0],ymdhms[1],ymdhms[2],doy, ymdhms[3],ymdhms[4],sec, mjd, soy);
  }
  buf = buf2;
}

// std::string Time_dat::Print_time(const int iptn) const
// {

//   switch (iptn){
//   case 0: // mjd 
//     sprintf(buf2, "%12.6lf ",mjd);
//     break;
//   case 10: // yyyy/ddd hh:mm:ss.mmm
//     sprintf(buf2, "%04d/%03d %02d:%02d:%06.3lf",
// 	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
//     break;
//   case 11: // YYYYyDDDdHHhMMmSS.SSS
//     sprintf(buf2, "%04dy%03dd%02dh%02dm%06.3lf",
// 	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
//     break;
//   case 12: // YYYY DDD HH MM SS
//     sprintf(buf2, "%04d %03d %02d %02d %02.0lf",
// 	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
//     break;
//   case 13: // YYYYDDDHHMMSS
//     sprintf(buf2, "%04d%03d%02d%02d%02.0lf",
// 	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
//     break;
//   case 14: // YYYY/DDD/HH:MM:SS
//     sprintf(buf2, "%04d/%03d/%02d:%02d:%02.0lf",
// 	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
//     break;
//   case 15: // YYYY/DDD/HH:MM:SS.mmm
//     sprintf(buf2, "%04d/%03d/%02d:%02d:%06.3lf",
// 	    ydhms[0],ydhms[1],ydhms[2], ydhms[3], sec);
//     break;
//   case 20: // yyyy/mm/dd hh:mm:ss.mmm 
//     sprintf(buf2, "%04d/%02d/%02d %02d:%02d:%06.3lf",
// 	    ymdhms[0],ymdhms[1],ymdhms[2],ymdhms[3],ymdhms[4],sec);
//     break;
//   case 21: // yyyy/mm/dd hh:mm:ss
//     sprintf(buf2, "%04d/%02d/%02d %02d:%02d:%02.0lf",
// 	    ymdhms[0],ymdhms[1],ymdhms[2],ymdhms[3],ymdhms[4],sec);
//     break;
//   default:
//     sprintf(buf2, "%04d/%02d/%02d (%07.3lf) %02d:%02d:%06.3lf mjd=%.5lf, soy=%.3lf",
// 	    ymdhms[0],ymdhms[1],ymdhms[2],doy, ymdhms[3],ymdhms[4],sec, mjd, soy);
//   }

//   char buf2[256];
//   Print_time(buf2,iptn);
//   std::string ret = buf2;
//   return ret;
// }

Time_dat Time_dat::operator+(const Time_dat& y) const {
  double soy = this->soy + y.soy, 
    soy_year = 86400.*365.;
  int uru=0, 
    yy = ydhms[0];
  
  if( (yy % 4 ==0 &&  yy % 100 !=0 ) ||  yy % 400 ==0) uru=1; else uru=0;
  while( soy >= soy_year + uru*86400){
    yy ++;
    soy -= (soy_year+uru*86400);
    if( (yy % 4 ==0 &&  yy % 100 !=0 ) ||  yy % 400 ==0) uru=1; else uru=0;
  }
  Time_dat z(yy,1,1,0,0,0);
  //  z.soy = soy;
  //  z.ydhms[0] = z.ymdhms[0] = yy;
  z.Sec2dhms(soy);
  z.Doy2md();
  z.Mjd_cal();
  return(z);
}


 Time_dat operator+(const Time_dat& x, const double dsec) {
  double soy = x.soy + dsec, soy_year = 86400.*365.;
  int uru=0, 
    yy = x.ydhms[0];
  
  if( (yy % 4 ==0 &&  yy % 100 !=0 ) ||  yy % 400 ==0) uru=1; else uru=0;
  while( soy >= soy_year + uru*86400.){
    yy ++;
    soy -= (soy_year + uru*86400);
    if( (yy % 4 ==0 &&  yy % 100 !=0 ) ||  yy % 400 ==0) uru=1; else uru=0;
  }
  Time_dat z(yy,1,1,0,0,0);
  //  z.soy = soy;
  //  z.ydhms[0] = z.ymdhms[0] = yy;
  z.Sec2dhms(soy);
  z.Doy2md();
  z.Mjd_cal();
  return(z);
}


/* Find Index for the mjd time */
int index_tarr(std::vector <Time_dat>& tarr, Time_dat t_ref){

  int i1,i2;
  i1 = i2 =0;
  int icnt=0;
  while(icnt < (int)tarr.size() &&
	i1*i2 ==0 ){
    i1=i2=0;
    if( tarr[icnt].Mjd() <= t_ref.Mjd()) i1=1;    
    if( tarr[icnt+1].Mjd() >= t_ref.Mjd()) i2=1;
    icnt++;
  }
  
  int iend = tarr.size()-1;
  if( i1 == 1 && i2 ==0){
    fprintf(stderr,"Error Specified time range of ephemeris (");
    tarr[0].Print_time(stderr,10); fprintf(stderr, ") - (");
    tarr[iend].Print_time(stderr,10); 
    fprintf(stderr, "is before the target date:");
    t_ref.Print_time(stderr,10); fprintf(stderr, "\n");
    return(-1);
  } 
  if( i1 == 0 && i2 ==1){
    fprintf(stderr,"Error Specified time range of ephemeris (");
    tarr[0].Print_time(stderr,10); fprintf(stderr, ") - (");
    tarr[iend].Print_time(stderr,10); 
    fprintf(stderr, "is after the target date:");
    t_ref.Print_time(stderr,10); fprintf(stderr, "\n");
    return(-1);
  }
  int iref = icnt;

  return(iref);

};

double Time_dat::TAI_UTC(void){

  const int n=37;
  double tai_utc_cff[n][4] = {
    { 2437300.5, 1.4228180, 37300., 0.001296},
    { 2437512.5, 1.3728180, 37300., 0.001296},
    { 2437665.5, 1.8458580, 37665., 0.0011232},
    { 2438334.5, 1.9458580, 37665., 0.0011232},
    { 2438395.5, 3.2401300, 38761., 0.001296},
    { 2438486.5, 3.3401300, 38761., 0.001296},
    { 2438639.5, 3.4401300, 38761., 0.001296},
    { 2438761.5, 3.5401300, 38761., 0.001296},
    { 2438820.5, 3.6401300, 38761., 0.001296},
    { 2438942.5, 3.7401300, 38761., 0.001296},
    { 2439004.5, 3.8401300, 38761., 0.001296},
    { 2439126.5, 4.3131700, 39126., 0.002592},
    { 2439887.5, 4.2131700, 39126., 0.002592},
    { 2441317.5, 10.0, 41317., 0.0},
    { 2441499.5, 11.0, 41317., 0.0},
    { 2441683.5, 12.0, 41317., 0.0},
    { 2442048.5, 13.0, 41317., 0.0},
    { 2442413.5, 14.0, 41317., 0.0},
    { 2442778.5, 15.0, 41317., 0.0},
    { 2443144.5, 16.0, 41317., 0.0},
    { 2443509.5, 17.0, 41317., 0.0},
    { 2443874.5, 18.0, 41317., 0.0},
    { 2444239.5, 19.0, 41317., 0.0},
    { 2444786.5, 20.0, 41317., 0.0},
    { 2445151.5, 21.0, 41317., 0.0},
    { 2445516.5, 22.0, 41317., 0.0},
    { 2446247.5, 23.0, 41317., 0.0},
    { 2447161.5, 24.0, 41317., 0.0},
    { 2447892.5, 25.0, 41317., 0.0},
    { 2448257.5, 26.0, 41317., 0.0},
    { 2448804.5, 27.0, 41317., 0.0},
    { 2449169.5, 28.0, 41317., 0.0},
    { 2449534.5, 29.0, 41317., 0.0},
    { 2450083.5, 30.0, 41317., 0.0},
    { 2450630.5, 31.0, 41317., 0.0},
    { 2451179.5, 32.0, 41317., 0.0},
    { 2453736.5, 33.0, 41317., 0.0}
  };

  int i;
  for(i=0; i<n; i++){
    if( mjd < tai_utc_cff[i][0]-2400000.5) break;
  }
  int ip = i-1;
  double xmjd = tai_utc_cff[ip][2];
  double tai_utc = tai_utc_cff[ip][1] + (mjd - xmjd)*tai_utc_cff[ip][3];
  return(tai_utc);
}

int Smonth(const std::string mon){
  int pos =-1;
  if( (pos = mon.find("Jan")) >= 0) return(1);
  if( (pos = mon.find("Feb")) >= 0) return(2);
  if( (pos = mon.find("Mar")) >= 0) return(3);
  if( (pos = mon.find("Apr")) >= 0) return(4);
  if( (pos = mon.find("May")) >= 0) return(5);
  if( (pos = mon.find("Jun")) >= 0) return(6);
  if( (pos = mon.find("Jul")) >= 0) return(7);
  if( (pos = mon.find("Aug")) >= 0) return(8);
  if( (pos = mon.find("Sep")) >= 0) return(9);
  if( (pos = mon.find("Oct")) >= 0) return(10);
  if( (pos = mon.find("Nov")) >= 0) return(11);
  if( (pos = mon.find("Dec")) >= 0) return(12);
  return -1;
}

bool Time_dat::Check_timedat(void){
  if(ymdhms[0] > 0 && ymdhms[0] < 9999 && 
     ymdhms[1] <13 &&  ymdhms[1] >0 &&  
     ymdhms[2] <32 &&  ymdhms[2] >=0 &&  
     ymdhms[3] <25 &&  ymdhms[3] >=0 &&  
     ymdhms[4] <60 &&  ymdhms[4] >=0 &&   
     ymdhms[5] <60 &&  ymdhms[5] >=0  ) 
    return true;
  else return false;
}

HY_Time Time_dat::Hy_time()const{

  int hy = (ydhms[0] - 2000)*2;
  double soy_hy = 0;
  if(ymdhms[1] > 7){ 
    hy ++; 
    Time_dat t0(ydhms[0],7,1,0,0,0);
    soy_hy = t0.Soy();
  }
   HY_Time t0(hy, soy-soy_hy);
   return t0;
}


time_t Time_dat::Time_t() const {
  struct tm tm1;
  tm1.tm_year = ymdhms[0] - 1900;
  tm1.tm_mon = ymdhms[1] -1;
  tm1.tm_mday = ymdhms[2];
  tm1.tm_hour = ymdhms[3];
  tm1.tm_min = ymdhms[4];
  tm1.tm_sec = ymdhms[5];
  setenv("TZ","GMT",1);tzset();
  time_t t2 = mktime(&tm1);
  return t2;
}

int Time_dat::Set_time(const time_t t1){
  struct tm tm1;
  gmtime_r(&t1, &tm1);
  Set_time(tm1.tm_year+1900, tm1.tm_mon+1,tm1.tm_mday, 
		 tm1.tm_hour, tm1.tm_min, tm1.tm_sec);
  return 0;
}

int Str2Tint(char *a){
  int scale=1;
  
  if (a[strlen(a)-1]=='h') {
    scale=60*60; /* Hours -> seconds  */
  } else if (a[strlen(a)-1]=='m') {
    scale=60; /* Minutes -> seconds  */
  }
  int length=(uint32_t) ceil(atof(a)*scale);
  return length;
}
