/** 
 * @Author M.Sekido
 * @Version $Rev: 201 $
 * @Date  $Date: 2016-08-28 22:44:01 +0900 (Sun, 28 Aug 2016) $
 * $Id: InVSI-bist.cpp 201 2016-08-28 13:44:01Z vlbi $ 
 */

#include "InVSI_mdl.hpp"
#include "bitutl.hpp"


char* me;

const char* version =  "$Rev: 201 $ Updated $Date: 2016-08-28 22:44:01 +0900 (Sun, 28 Aug 2016) $ by $Author: vlbi $";

void LogSetup(const int verbose);

void usage(){
  cerr <<  "Usage: " << me << "[-h] [-n <avg npt( x1000 points)> ] [-b <bit>]" << endl; 
  cerr << " Program to check the Bit histgram of specified quantization bit. \n";
  cerr << " -b : Quantization bit (default=2) \n";
  cerr << " -n : Number of Average points. (default 10k) \n";
  cerr << " -h : Help \n";
  //  cerr << " -v : Verbose Level 0-4.\n";
  cerr << " " << version << std::endl;
  exit(1); 

}

int main(int argc, char* argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  int verbose_level =0;
  int k=0;
  int navg_k = 10;
  int qbit=2;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hn:v:b:")) { 
    case 'v':  verbose_level = atoi(optarg);   break;
    case 'n':   navg_k = atoi(optarg);   break;
    case 'b':   qbit = atoi(optarg);   break;
    case 'h':   
    case '?':    usage(); break;
    }
  }

  ulong navg = navg_k * 1000 ;
  string Stname("Kashima34");
  string Stid("kb");

  DAS_Info indas( 64000000, 16, 2 );
  LogSetup(verbose_level);
   
  K5_VSI *k5vsi = new K5_VSI;
  InDev_Model *input_mdl = k5vsi;
  VLBI_Frame* in_vlbi_frame = new K5VSI_Frame;
  in_vlbi_frame->Set_DasInfo( indas );

  input_mdl->Set_VlbiFrame(in_vlbi_frame);
  input_mdl->Set_StnameID(Stname.c_str(),Stid.c_str());

  if( input_mdl->OpenDev(InDev_Model::MONIT) < 0){
    cerr << "E K5_VSI::Ready() failed\n";
    exit(1);
  };
  
  uint32_t pagesize = input_mdl->PageLength();
  int npage = input_mdl->Npage();
  printf("Number of Page = %d pages\n", npage );
  printf("Pagesize = %u MByte\n", pagesize*4/1000000 );
  int first_page = input_mdl->FirstPage();
  printf("PgStat=[%s]\n", input_mdl->StrPageState(1).c_str());

  int ipage = (first_page + npage-1) % npage;
  time_t page_epoch = k5vsi->GetFrameEpoch(ipage);
  struct tm tm1;
  char dumbuf[256];
  gmtime_r ( &page_epoch, &tm1 );
  strftime ( dumbuf,256,"%Y/%j-%H:%M:%S",&tm1 );
  printf("FirstPage = %d ipage=%d, Epoch=%s\n", 
	 first_page, ipage, dumbuf);

  //  time_t t1;
  //  uint32_t ClockMHz = input_mdl->PageLength()/1000000;
  const uint8_t* mapped_page(0);
  
  Bit_hist b_hist(qbit);
  int nch = 32/qbit;
  double dt0 = GetTime();


  while(1){
    mapped_page = k5vsi->MappingData(page_epoch);    
    if(mapped_page == 0){
      gmtime_r ( &page_epoch, &tm1 );
      strftime ( dumbuf,256,"%Y/%j-%H:%M:%S",&tm1 );
      fprintf(stderr, "E Failed in mapping (%s)\n", dumbuf);
    break;
    }
    uint32_t* ptr32 = (uint32_t*) mapped_page;
    
    for(ulong i=0; i< navg; i++){
      b_hist.Count(*ptr32); 
      ptr32++;
    }

    k5vsi->UnMapping(page_epoch);    
    cout << "================================================\n";
    cout << setw(3) << " ch" << b_hist.StrHist(-1) << endl;
    for(int i=0; i< nch; i++){
      cout << setw(3) << i+1 << b_hist.StrHist(i) << endl;
    }
    b_hist.Init(qbit); 
    

//     t1 = k5vsi->GetDevEpoch();
//     gmtime_r ( &t1, &tm1 );
//     strftime ( dumbuf,256,"%Y/%j-%H:%M:%S",&tm1 );
//     printf("DeviceEpoch=%lu(%s)  Clock=%lu MHz (%lu Mbps)\n", 
// 	   t1, dumbuf, (ulong)ClockMHz, (ulong)ClockMHz * 32);

//     time(&t1);
//     gmtime_r ( &t1, &tm1 );
//     strftime ( dumbuf,256,"%Y/%j-%H:%M:%S",&tm1 );
//     printf("[%s]FstPg=%d: ", dumbuf,input_mdl->FirstPage());
//     printf("PgStat=[%s]\n", input_mdl->StrPageState(1).c_str());
    dt0 += 1.0;
    double dt1 = GetTime();
    unsigned long usec = (ulong)((dt0 - dt1)*1000000);
    usleep(usec);
    page_epoch++;
  }


}


void LogSetup(const int verbose=-1){

  string logdir = "/tmp/";
  string error_file = logdir + me +"_error.txt";
   
  string debug_file = logdir + me + "_dbg.txt";
  Logsystem::debug_logger()->set_stream(debug_file);
  string info_file = logdir + me + "_info.txt";
  Logsystem::info_logger()->set_stream(info_file);
  string warn_file = logdir + me + "_warn.txt";
  Logsystem::warning_logger()->set_stream(warn_file);
  string err_file = logdir + me + "_err.txt";
  Logsystem::error_logger()->set_stream(err_file);
      
  switch(verbose){
  case -1:
    cerr << " No Logging." << endl;
    break;
  case 0:
    Logsystem::debug_logger()->enable_logging(Logger::MINIMUM); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->set_stream(error_file);
    Logsystem::error_logger()->enable_logging();
    break;
  case 1:
    Logsystem::debug_logger()->enable_logging(Logger::SMALL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 2:
    Logsystem::debug_logger()->enable_logging(Logger::MEDIUM); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 3:
    Logsystem::debug_logger()->enable_logging(Logger::DETAIL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 4:
  default:
    Logsystem::debug_logger()->enable_logging(Logger::ALL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  }
  
}
