/** 
 * @Version $Rev: 132 $
 * @Author M.Sekido
 * @Date  $Date: 2014-07-25 14:19:07 +0900 (Fri, 25 Jul 2014) $
 * $Id: InVSI-test.cpp 132 2014-07-25 05:19:07Z vlbi $ 
 */

#include "InVSI_mdl.hpp"

char* me;
const char* version =  "$Rev: 132 $ Updated $Date: 2014-07-25 14:19:07 +0900 (Fri, 25 Jul 2014) $ by $Author: vlbi $";

void LogSetup(const int verbose);

void usage(){
  cerr <<  "Usage: [-h] [-v <n>]" << me 
       << " Testing InVSI-model.\n";
  cerr << " -h : Help \n";
  cerr << " -v : Verbose Level 0-4.\n";
  cerr << " " << version << std::endl;
  exit(1); 

}

int main(int argc, char* argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  int verbose_level =0;
  int k=0;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hv:")) { 
    case 'v':
      verbose_level = atoi(optarg);
      break;
    case 'h':   
    case '?':    usage(); break;
    }
  }

  string Stname("Kashima34");
  string Stid("kb");

  DAS_Info indas( 64000000, 16, 2 );
  LogSetup(verbose_level);
   
  K5_VSI *k5vsi = new K5_VSI;
  InDev_Model *input_mdl = k5vsi;
  VLBI_Frame* in_vlbi_frame = new K5VSI_Frame;
  in_vlbi_frame->Set_DasInfo( indas );

  input_mdl->Set_VlbiFrame(in_vlbi_frame);
  input_mdl->Set_StnameID(Stname.c_str(),Stid.c_str());

  input_mdl->Ready();
  input_mdl->start(); // reading data Started 
  
  uint32_t pagesize = input_mdl->PageLength();
  int npage = input_mdl->Npage();
  printf("Number of Page = %d pages\n", npage );
  printf("Pagesize = %u MByte\n", pagesize*4/1000000 );
  printf("FirstPage = %d \n", input_mdl->FirstPage());

  char dumbuf[256];
  struct tm tm1;
  time_t t1;
  uint32_t ClockMHz = input_mdl->PageLength()/1000000;

  while(1){
    t1 = k5vsi->GetDevEpoch();
    gmtime_r ( &t1, &tm1 );
    strftime ( dumbuf,256,"%Y/%j-%H:%M:%S",&tm1 );
    printf("DeviceEpoch=%lu(%s)  Clock=%lu MHz (%lu Mbps)\n", 
	   t1, dumbuf, (ulong)ClockMHz, (ulong)ClockMHz * 32);

    time(&t1);
    gmtime_r ( &t1, &tm1 );
    strftime ( dumbuf,256,"%Y/%j-%H:%M:%S",&tm1 );
    printf("[%s]FstPg=%d: ", dumbuf,input_mdl->FirstPage());
    printf("PgStat=[%s]\n", input_mdl->StrPageState(1).c_str());
    usleep(1000000);
  }


}


void LogSetup(const int verbose=-1){

  string logdir = "/tmp/";
  string error_file = logdir + me +"_error.txt";
   
  string debug_file = logdir + me + "_dbg.txt";
  Logsystem::debug_logger()->set_stream(debug_file);
  string info_file = logdir + me + "_info.txt";
  Logsystem::info_logger()->set_stream(info_file);
  string warn_file = logdir + me + "_warn.txt";
  Logsystem::warning_logger()->set_stream(warn_file);
  string err_file = logdir + me + "_err.txt";
  Logsystem::error_logger()->set_stream(err_file);
      
  switch(verbose){
  case -1:
    cerr << " No Logging." << endl;
    break;
  case 0:
    Logsystem::debug_logger()->enable_logging(Logger::MINIMUM); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->set_stream(error_file);
    Logsystem::error_logger()->enable_logging();
    break;
  case 1:
    Logsystem::debug_logger()->enable_logging(Logger::SMALL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 2:
    Logsystem::debug_logger()->enable_logging(Logger::MEDIUM); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 3:
    Logsystem::debug_logger()->enable_logging(Logger::DETAIL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  case 4:
  default:
    Logsystem::debug_logger()->enable_logging(Logger::ALL); 
    Logsystem::info_logger()->enable_logging();
    Logsystem::error_logger()->enable_logging();
    Logsystem::warning_logger()->enable_logging();
    break;
  }
  
}
